import * as i0 from '@angular/core';
import { Injectable, EventEmitter, Directive, Input, Output, NgModule, forwardRef, ViewChild, Component, InjectionToken, Inject } from '@angular/core';
import * as i1$1 from '@angular/common';
import { DatePipe, CommonModule } from '@angular/common';
import { Subject, Subscription, BehaviorSubject, from, of, lastValueFrom } from 'rxjs';
import * as i1 from '@angular/common/http';
import { HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { FormControl, FormBuilder, FormGroup, Validators, NG_VALUE_ACCESSOR } from '@angular/forms';
import * as i3 from '@angular/router';
import { map, catchError, tap } from 'rxjs/operators';

class ServiwUtil {
    constructor() {
    }
    /**
     * Retornar un mes anterior al actual
     *
     * @param mes
     */
    mesAnterior(mes) {
        if (mes <= 1)
            mes = 12;
        else
            mes--;
        return mes;
    }
    /**
     * Calcula y retorna un año anterior, según el mes
     *
     * @param ano
     * @param mes
     */
    anoAnterior(ano, mes) {
        if (mes == 1)
            ano--;
        return ano;
    }
    /**
     * Asignar datos de un array a una interfaz dada, donde distribuye los datos
     * en secuencia de array a la interfaz.
     *
     * 1. Recorre cada tipo de dato en interfaz
     *  1.1. Obtiene el valor y el tipo de dato
     *  1.2. Refactoriza objects de la interfaz
     *  1.3. Establece el valor asignado
     *
     * @param interfaz Object parametriza tipo y secuencia de datos
     * @param data Datos a asignar en la interfaz
     * @param i Inicio lectura de datos en array
     * @default i = 0
     */
    refactorInterfaz(interfaz, data, i = 0) {
        //  1.
        for (let name in interfaz) {
            // 1.1.  
            const objData = interfaz[name];
            const value = objData;
            const type = objData instanceof Object && !(objData instanceof Date);
            if (type)
                // 1.2.
                i = this.refactorInterfaz(objData, data, i);
            else if (value == -1 || value == '' || objData instanceof Date) {
                // 1.3.
                interfaz[name] = this.getDataType(data[i], objData);
                i++;
            }
        }
        return i;
    }
    /**
     * Define el tipo de dato a retornar
     *
     * 1. Define el tipo de dato
     * 2. Realiza la conversión al dato asignado
     *
     * @param data Dato a convertir
     * @param type type Define el tipo de dato a retornar
     * @description Date | Object | string | number | boolean, Donde se asigna
     * Cualquier valor en (type) que define el tipo de dato a retornar
     * @returns data
     */
    getDataType(data, type) {
        //  1.
        const typeI = typeof (type);
        // 2.
        if (type instanceof Date)
            data = new Date(data);
        else if (type instanceof Object || typeI == 'object')
            data = Object.assign({});
        else if (typeI === 'string')
            data = data + '';
        else if (typeI === 'number')
            data = Number.parseInt(data);
        else if (typeI === 'boolean')
            if (type)
                data = true;
            else
                data = false;
        return data;
    }
    /**
     * Determines if value is objet
     * @param value to evaluate
     * @returns
     */
    static isObject(value) {
        return value != null && typeof value === 'object';
    }
    static isString(value) {
        return typeof value === 'string';
    }
    static isNumber(value) {
        return typeof value === 'number';
    }
    //#region  Functions static
    /**
     * Clones object
     * @param object
     * @returns object
     */
    static cloneObject(object) {
        const clone = { ...object };
        for (let key in object) {
            const val = object[key];
            if (val instanceof Date)
                clone[key] = new Date(val);
            else if (ServiwUtil.isObject(val))
                clone[key] = this.cloneObject(val);
        }
        return clone;
    }
    /**
     * Validar si los valores de una propiedad son nulos o indefinidos.
     * También valida si el numero de propiedades corresponde al número
     * asignado en parámetro.
     *
     * 1. Valida numero de propiedades
     * 2. Valida el valor de cada propiedad
     *
     * @param object interfaz a validar
     * @param numbers numero de propiedades en interfaz original
     */
    static validPropierties(object, numbers) {
        if (numbers) {
            // 1.
            const num = Object.keys(object).length;
            if (numbers != num)
                return false;
        }
        for (var prop in object) {
            // 2.
            if (object[prop] === undefined || object[prop] === null)
                return false;
        }
        return true;
    }
    /**
     * Emitir cambios de una suscripción #addEditCall para actualizar o agregar
     * un objeto en una lista (Array)
     *
     * 1. Validar elementos de la suscripción
     * 2. Declarar variables
     * 3. Recorre cada dato de un array
     *  3.1. Compara identificadores de cada objeto
     *  3.2. Actualiza un valor de la tabla
     *  3.3. Agrega valores actuales
     * 4. Agrega un nuevo elemento
     * 5. Emitir valores y reseter datos
     *
     * @param array Array
     * @param addUpdate Objecto a actualizar
     * @param id  @default{id} {Id} Identificador
     * @param esquemaId Propiedades a comparar id tipo object
     * @param remove Dato/s a remover en un array
     * @param unshift Agregar items al inicio o final
     */
    static updateArray(array, addUpdate, id = 'id', esquemaId, remove, unshift = true) {
        // 1.
        array = array ? [...array] : [];
        // 2.
        const values = Array.isArray(addUpdate) ? addUpdate : [addUpdate];
        let actualizado = false;
        // 3.
        for (let i = 0; i < values.length; i++) {
            const value = values[i];
            const index = ServiwUtil.searchIndex(value, array, id, esquemaId);
            if (index >= 0)
                array[index] = value;
            else if (unshift)
                array.unshift(value);
            else
                array.push(value);
        }
        // 4.
        if (remove)
            array = this.removeArray(remove, array, id, esquemaId);
        return array;
    }
    /**
     * Remover uno o mas elementos de un array
     *
     * 1. Validar elementos a remover
     * 2. Define array a remover
     * 3. Elimina cada elemento de la lista
     *
     * @param remove Elemento o lista a remover
     * @param vals Array de consulta
     * @param id Identificador para objetcs
     * @param esquemaId Esquema Id personalizado
     */
    static removeArray(remove, vals, id = 'id', esquemaId) {
        // 1.   
        if (!remove)
            return vals;
        // 2.
        const array = remove;
        const removes = Array.isArray(remove) ? array : [remove];
        for (let j = 0; j < removes.length; j++) {
            // 3.
            const rem = removes[j];
            const index = this.searchIndex(rem, vals, id, esquemaId);
            if (index >= 0)
                vals.splice(index, 1);
        }
        return [...vals];
    }
    /**
     * Consultar index de un elemento en un array
     *
     * @param val Elemento en array
     * @param vals Array de consulta
     * @param id Identificador para objetcs
     * @param esquemaId Esquema Id personalizado
     */
    static searchIndex(val, vals, id = 'id', esquemaId) {
        // 1.
        vals = vals || [];
        val = val || Object.assign({});
        // 2.
        const search = (elm, index, array) => {
            const val1 = id ? val[id] : val;
            const val2 = id ? elm[id] : elm;
            const equals = ServiwUtil.objEquals(val1, val2, esquemaId);
            return equals;
        };
        return vals.findIndex(search);
    }
    /**
     * Validar si dos objetos son iguales.
     *
     * 1. Validar datos diferentes a nulos
     * 2. Interfaz Datos a comparar
     * 3. Establecer datos por esquema personalziado
     * 4. Comparar objetos
     *
     * @param obj1 objeto a comparar
     * @param obj2 objeto a comparar
     * @param esquemaId Esquema personalizado a comparar
     */
    static objEquals(obj1, obj2, esquema) {
        // 1.
        const valid1 = ServiwUtil.empty(obj1);
        const valid2 = ServiwUtil.empty(obj2);
        if (valid1 != valid2)
            return false;
        if (valid1 && valid2)
            return true;
        // 2.
        let objs;
        if (esquema) {
            // 3.
            const e1 = ServiwUtil.getEsquema(esquema, obj1);
            const e2 = ServiwUtil.getEsquema(esquema, obj2);
            objs = { obj1: e1, obj2: e2 };
        }
        else
            objs = { obj1, obj2 };
        // 4.
        const objJson1 = JSON.stringify(objs.obj1);
        const objJson2 = JSON.stringify(objs.obj2);
        return objJson1 === objJson2;
    }
    /**
     * Logs serviw util
     * @param data
     */
    static logs(...data) {
        for (const c of data)
            console.log(c);
    }
    /**
     * Clonar datos de un objeto (data) hacia un esquema (key)
     *
     * 1. Inicializar datos de respuesta
     * 2. Establecer datos por propiedad
     *
     * @param key Esquema a clonar de data
     * @param data Objecto original
     */
    static getEsquema(key, data) {
        // 1.
        let dateKey = Object.assign({});
        const getData = data;
        for (let name in key) {
            // 2.
            const objKey = key[name];
            const value = getData[name];
            const valueData = ServiwUtil.isObject(value) && ServiwUtil.isObject(objKey) ? ServiwUtil.getEsquema(objKey, value) : value;
            dateKey[name] = valueData;
        }
        return dateKey;
    }
    /**
     * Emptys basic
     * @param data
     * @returns true if basic
     */
    static isEmpty(data) {
        if (data == null || data == undefined)
            return true;
        return false;
    }
    /**
     * Params serviw util
     * @param data validate
     * @param [omit] data == value  return false
     * @returns true if data = ( null || undefined || '' || {} || [] || !data )
     * @description || -> o
     */
    static empty(data, omit) {
        let dataEmpty = false;
        if (data == null || data == undefined)
            dataEmpty = true;
        else if (ServiwUtil.isString(data) && data.trim() === '')
            dataEmpty = true;
        else if (ServiwUtil.isObject(data) && Object.keys(data).length == 0)
            dataEmpty = true;
        else if (Array.isArray(data) && data.length == 0)
            dataEmpty = true;
        else if (!data)
            dataEmpty = true;
        if (!dataEmpty && omit)
            return !this.objEquals(data, omit);
        return dataEmpty;
    }
    /**
     * Valids date
     * @param date
     * @param [today]
     * @returns date of date
     */
    static validDate(date, today = true) {
        if (date) {
            date = new Date(date);
        }
        else if (today) {
            date = new Date();
        }
        return date;
    }
    /**
     * Implement datePipe
     *
     * @param value
     * @param format
     * @param timezone
     * @param locale
     */
    static datePipe(value, format, timezone, locale) {
        const datePipe = new DatePipe('en-ES');
        return datePipe.transform(value, format, timezone, locale);
    }
    ;
    /**
     * Consultar el valor de la ultima propiedad asignada en un object
     *
     * 1. Obtener copias del objeto actal
     * 2. Separar las propiedades por caracter (.)
     * 3. Consulta y filtra datos de las propiedades
     *
     * @param filter
     */
    static valueProperties(item, properties) {
        // 1.
        let property = item;
        // 2.
        const objects = properties.split('.');
        // 3.
        for (let name of objects)
            if (property)
                property = property[name];
        return property;
    }
    /**
     *
     * @param quantity
     */
    static numberLength(quantity) {
        return quantity;
    }
    /**
     * Primera letra a mayuscula
     * @param text
     */
    static firstCapitalLetter(text) {
        return text.charAt(0).toUpperCase() + text.slice(1);
    }
    /**
     * Primera letra a mayuscula
     * @param text
     */
    static firstLowercaseLetter(text) {
        return text.charAt(0).toLowerCase() + text.slice(1);
    }
    /**
     * Sets key value
     * @param object
     * @param keyValue
     * @returns key value
     */
    static setKeyValue(object, keyValue) {
        const data = {};
        for (const name in object)
            data[name] = object[name][keyValue];
        return data;
    }
    /**
     * Resets object
     * @param obj
     */
    static resetObject(obj) {
        for (const name in obj) {
            delete obj[name];
        }
        return obj;
    }
    /**
     * Returns serviw util
     * @returns object {}
     */
    static emptyObject() {
        return {};
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwUtil.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwUtil, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwUtil.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwUtil, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwUtil, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });

var TYPES;
(function (TYPES) {
    TYPES["string"] = "string";
    TYPES["date"] = "date";
    TYPES["int"] = "int";
    TYPES["float"] = "float";
    TYPES["double"] = "double";
    TYPES["long"] = "long";
    TYPES["boolean"] = "boolean";
})(TYPES || (TYPES = {}));
const SCHEMA_GRAPHQL = '[GRAPHQL]';
const SCHEMA_REST = '[REST]';
const MANY_TO_ONE = 'manyToOne';
const MANY_TO_MANY = 'manyToMany';
const ONE_TO_ONE = 'oneToOne';
const ONE_TO_MANY = 'oneToMany';
const OPEN_SERVIW_FORM = '[OPEN]:ServiwCoreForm<I>';

class SettingsServiwCore {
    /**
     * Creates an instance of settings serviw core.
     * @param resolver
     */
    constructor(resolver) {
        this.resolver = resolver;
        this._setConfig = new EventEmitter();
        this._config = {
            formTemplate: null,
            settingsScheme: { api: '/', endpointGraphql: 'api/graphql', scheme: SCHEMA_GRAPHQL, entityFieldSave: true, nameApp: 'APP' },
            resolver,
            options: null,
            tableTemplate: { template: null }
        };
        this.emitConfig();
    }
    /**
     * Gets whether on update
     */
    get onUpdate() {
        return this._setConfig.asObservable();
    }
    /**
     * Gets form template
     */
    get formTemplate() {
        return this._config.formTemplate;
    }
    /**
     * Sets form template
     */
    set formTemplate(components) {
        this._config.formTemplate = components;
        this.emitConfig();
    }
    /**
     * Gets ng module
     */
    get ngModule() {
        return this._config.ngModule;
    }
    /**
     * Sets ng module
     */
    set ngModule(ngModule) {
        this._config.ngModule = ngModule;
        this.emitConfig();
    }
    /**
     * Sets base url
     */
    set baseUrl(data) {
        this._config.settingsScheme = Object.assign(this._config.settingsScheme, data);
        this.emitConfig();
    }
    /**
     * Gets base url
     */
    get API() {
        return this._config.settingsScheme.api;
    }
    /**
     * Gets api graphql
     */
    get API_GRAPHQL() {
        return this._config.settingsScheme.api + this._config.settingsScheme.endpointGraphql;
    }
    /**
     * Gets name app
     */
    get NAME_APP() {
        return this._config.settingsScheme.nameApp;
    }
    /**
     * Gets scheme
     */
    get scheme() {
        return this._config.settingsScheme.scheme;
    }
    /**
     * Gets entity field save
     */
    get entityFieldSave() {
        return this._config.settingsScheme.entityFieldSave;
    }
    /**
     * Sets http options
     */
    set httpOptions(val) {
        if (!ServiwUtil.empty(val)) {
            this._config.options = val;
            this.emitConfig();
        }
    }
    /**
     * Gets http options
     */
    get httpOptions() {
        return this._config.options;
    }
    /**
     * Gets table template
     */
    get tableTemplate() {
        return this._config.tableTemplate;
    }
    /**
     * Sets table template
     */
    set tableTemplate(val) {
        this._config.tableTemplate = val;
        this.emitConfig();
    }
    /**
     * Emits config
     */
    emitConfig() {
        this._setConfig.emit(this._config);
    }
}
/** @nocollapse */ /** @nocollapse */ SettingsServiwCore.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: SettingsServiwCore, deps: [{ token: i0.ComponentFactoryResolver }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ SettingsServiwCore.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: SettingsServiwCore, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: SettingsServiwCore, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: i0.ComponentFactoryResolver }]; } });

class ServiwRest {
    /**
     * Creates an instance of serviw rest.
     * @param http
     */
    constructor(http, settings) {
        this.http = http;
        this.settings = settings;
        this._getHttp = new Subject();
    }
    /**
     * Gets options
     */
    get options() {
        return this.settings.httpOptions || ServiwRest.optionsDefult();
    }
    /**
     * Get Observable http query
     */
    get getHttp() {
        return this._getHttp.asObservable();
    }
    /**
     * Gets serviw rest
     * @param link
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns get
     */
    get(link, params = {}, refactor, options = this.options) {
        // 1.
        this._getHttp.next('get');
        const _options = options;
        if (!ServiwUtil.empty(params))
            link += this.getLinkParameters(params);
        const observable = this.http.get(link, _options);
        const promise = this.refactorObservable(observable, refactor);
        return promise;
    }
    /**
     * Posts serviw rest
     * @param link
     * @param body
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns
     */
    post(link, body, params = {}, refactor, options = this.options) {
        this._getHttp.next('post');
        const _options = options;
        if (!ServiwUtil.empty(params))
            link += this.getLinkParameters(params);
        const observable = this.http.post(link, body, _options);
        const promise = this.refactorObservable(observable, refactor);
        return promise;
    }
    /**
     * Puts serviw rest
     * @param link
     * @param body
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns
     */
    put(link, body, params = {}, refactor, options = this.options) {
        this._getHttp.next('put');
        const _options = options;
        if (!ServiwUtil.empty(params))
            link += this.getLinkParameters(params);
        const observable = this.http.put(link, body, _options);
        const promise = this.refactorObservable(observable, refactor);
        return promise;
    }
    /**
     * Deletes serviw rest
     * @param link
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns
     */
    delete(link, params = {}, refactor, options = this.options) {
        this._getHttp.next('delete');
        const _options = options;
        if (!ServiwUtil.empty(params))
            link += this.getLinkParameters(params);
        const observable = this.http.delete(link, _options);
        const promise = this.refactorObservable(observable, refactor);
        return promise;
    }
    /**
     * Refactors observable
     * @param observable
     * @param refactor
     * @returns observable
     */
    refactorObservable(observable, refactor) {
        const func = (resolve, reject) => {
            const dataOk = (data) => {
                const refactorData = this.refactorData(data, refactor);
                resolve(refactorData);
            };
            const dataError = (error) => {
                reject(error);
            };
            observable.subscribe(dataOk, dataError);
        };
        const promise = new Promise(func);
        return promise;
    }
    /**
     * Refactors data
     * @param response
     * @param refactor
     * @returns data
     */
    refactorData(response, refactor) {
        let data = response;
        try {
            if (response.json)
                data = response.json();
            if (refactor)
                data = refactor(data);
        }
        catch (e) {
            data = response;
        }
        return data;
    }
    /**
     * Gets link parameters
     * @param data
     * @returns link parameters
     */
    getLinkParameters(data) {
        return ServiwRest.getLinkParameters(data);
    }
    // FILTER 
    /**
   * Filters serviw rest
   * @param link extends GenericEntityFilter<T>.filter
   * @param event table
   * @param [omitPage] omit pagination = false
   * @returns Promise<T>
   */
    filter(link, event, omitPage = false, refactor) {
        this._getHttp.next('filter');
        const data = this.refactorFilterBy(event, omitPage);
        link = link.endsWith('/') ? link + 'filter' : link + '/filter';
        return this.get(link, data, refactor);
    }
    /**
     * Counts filter
     * @param link GenericEntityFilter<T>.countFilter
     * @param event
     * @returns count filter
     */
    countFilter(link, event) {
        this._getHttp.next('countFilter');
        const data = this.refactorFilterBy(event, true);
        const path = 'count/filter';
        link = link.endsWith('/') ? link + path : link + '/' + path;
        return this.get(link, data);
    }
    /**
     * Refactors filter by
     * @param event
     * @param [omitPage]
     * @returns
     */
    refactorFilterBy(event, omitPage = false) {
        const refactorFilters = this.filtersOmit(this.refactorFilters(event, true)) || {};
        const data = {};
        let search = JSON.stringify(refactorFilters) || '';
        if (search && search.trim() != '') {
            search = search.replace(/,/g, ';');
            search = search.replace(/:/g, '==');
            search = search.replace(/"|{|}/g, '');
            if (search.trim() != '')
                data['search'] = search.trim();
        }
        if (!omitPage) {
            data['offset'] = event.first;
            data['rows'] = event.rows;
            const order = event.sortOrder == 1 ? '-ASC' : '-DESC';
            if (event.sortField)
                data['orderby'] = event.sortField + order;
        }
        return data;
    }
    /**
     * Establecer parametros a filtrar en una consulta slice.
     *
     * 1. Almacena datos a filtrar
     * 2. Establece filtros asignados en una tabla
     * 3. Establece filtros personalizados
     * 4. Retorna filtros para consultas  @function count()
     * 5. Establece datos para @var PageRequest
     * 6. Define columna y el tipo a ordenar
     *
     * @param event Datos de una consulta
     * @param count Consultas count
     */
    refactorFilters(event, count = false) {
        // 1.
        const refactorFilter = {};
        // 2.
        const filter = event.filters;
        for (const field in filter)
            if (undefined == filter[field] || null == filter[field])
                continue;
            else if (ServiwUtil.isObject(filter) && filter.hasOwnProperty(field) && filter[field].hasOwnProperty('value'))
                refactorFilter[field] = filter[field]['value'];
            else
                refactorFilter[field] = filter[field];
        // 3.
        if (event.globalFilter && event.globalFilter.trim() !== '')
            refactorFilter.globalFilter = event.globalFilter;
        else
            delete refactorFilter.globalFilter;
        if (count)
            return refactorFilter;
        // 5.
        refactorFilter.offset = event.first;
        refactorFilter.rows = event.rows;
        // 6.
        const order = event.sortOrder == 1 ? '-ASC' : '-DESC';
        if (event.sortField)
            refactorFilter.orderby = event.sortField + order;
        return refactorFilter;
    }
    /**
     * Filters omit
     * @param filters
     * @returns omit
     */
    filtersOmit(filters) {
        const pageable = ['offset', 'rows'];
        if (filters)
            for (const name of pageable) {
                delete filters[name];
            }
        ;
        return filters;
    }
    /**
     * Gets link parameters
     * @param data
     * @returns link parameters
     */
    static getLinkParameters(data) {
        let query = false;
        let url = '';
        for (const x in data) {
            if (data.hasOwnProperty(x) && (ServiwUtil.isNumber(data[x]) || (undefined !== data[x] && null !== data[x]))) {
                if (!query) {
                    url += '?';
                    query = true;
                }
                else {
                    url += '&';
                }
                url += x + '=' + data[x];
            }
        }
        return url;
    }
    /**
     * Options defult
     * @returns defult
     */
    static optionsDefult() {
        // 1
        const header = {
            'Content-Type': 'application/json'
        };
        // 2.
        const headers = new HttpHeaders(header);
        // 3.
        const options = {
            headers: headers,
            responseType: 'json'
            // withCredentials: true
        };
        return options;
    }
    /**
     * Bads request
     * @returns request
     */
    static BAD_REQUEST() {
        return 400;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwRest.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwRest, deps: [{ token: i1.HttpClient }, { token: SettingsServiwCore }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwRest.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwRest, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwRest, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: i1.HttpClient }, { type: SettingsServiwCore }]; } });

class ServiwValidator {
    static greaterThan(object, field) {
        const call = (control) => {
            const value = control.value;
            if (object.hasOwnProperty(field) && value > object[field]) {
                return null;
            }
            else {
                return { invalid: value };
            }
        };
        return call;
    }
    /**
     * Comparar el valor actual sea mayor o igual al valor del control
     * asignado como parametro nameControl, del FormGroup actual
     *
     * @param nameControl
     * @param add valor adiciona de diferencia
     */
    static minIn(nameControl, add = 0) {
        const call = (control) => {
            const form = control.parent; // Only FormGroup is used in this library
            if (!form || (form && !form.dirty))
                return null;
            const controlIn = form.controls[nameControl]; // Only serviwControl is used in this library
            const value = !isNaN(control.value) ? Number.parseFloat(controlIn.value) + add : 0;
            if (control.value)
                controlIn.addUnqValidators(ServiwValidator.max(control.value));
            if (!controlIn.valid || (control.value >= value))
                return null;
            else {
                return { min: { min: value } };
            }
        };
        return call;
    }
    /**
     * Comparar el valor actual sea menor al valor del control
     * asignado como parametro nameControl, del FormGroup actual
     *
     * @param nameControl
     * @param add valor adiciona de diferencia
     */
    static maxIn(nameControl, add = 0) {
        const call = (control) => {
            const form = control.parent; // Only FormGroup is used in this library
            if (!form || (form && !form.dirty))
                return null;
            const controlIn = form.controls[nameControl]; // ServiwControl must be used in this library
            const value = !isNaN(control.value) ? Number.parseFloat(controlIn.value) + add : 0;
            if (control.value)
                controlIn.addUnqValidators(ServiwValidator.min(control.value));
            if (!controlIn.valid || (control.value <= value))
                return null;
            else {
                return { max: { max: value } };
            }
        };
        return call;
    }
    static min(min) {
        const call = (control) => {
            let value = control.value;
            if (value >= min) {
                return null;
            }
            else {
                return { min: { min } };
            }
        };
        return call;
    }
    static max(max) {
        const call = (control) => {
            let value = control.value;
            if (value <= max) {
                return null;
            }
            else {
                return { max: { max } };
            }
        };
        return call;
    }
    /**
     * Validar fecha a ingresar sea menor o igual al valor asignado por parametro
     * @param max Fecha máxima
     * @param format formato de error
     * @default format 'dd / MMM / yyyy'
     */
    static maxDate(max, format = 'dd / MMM / yyyy') {
        const call = (control) => {
            const value = new Date(control.value);
            if (value <= max) {
                return null;
            }
            else {
                return { other: 'Fecha máxima a ingresar: ' + ServiwUtil.datePipe(max, format) };
            }
        };
        return call;
    }
    /**
     * Mins date in
     * @param nameControl
     * @returns
     */
    static minDateIn(nameControl) {
        const call = (control) => {
            const form = control.parent; //Only FormGroup is used in library
            if (!form || (form && !form.dirty))
                return null;
            const controlIn = form.controls[nameControl]; // Only ServiwControl is used in library
            if (controlIn.value)
                controlIn.addUnqValidators(ServiwValidator.maxDate(control.value));
            return ServiwValidator.minDate(controlIn.value)(control);
        };
        return call;
    }
    /**
     * Maxs date in
     * @param nameControl
     * @returns
     */
    static maxDateIn(nameControl, format = 'dd / MMM / yyyy') {
        const call = (control) => {
            const form = control.parent; // Only FormGroup is used in this libraru
            if (!form || (form && !form.dirty))
                return null;
            const controlIn = form.controls[nameControl]; // ServiwControl must be used in this library
            if (controlIn.value)
                controlIn.addUnqValidators(ServiwValidator.min(control.value));
            return ServiwValidator.minDate(controlIn.value, format)(control);
        };
        return call;
    }
    /**
     * Validar fecha a ingresar sea mayor o igual al valor asignado por parametro
     * @param min  Fecha mínima
     * @param [format] 'dd / MMM / yyyy'
     * @param [error] Fecha mínima a ingresar: {value}
     * @returns  call
     */
    static minDate(min, format = 'dd / MMM / yyyy', error = 'Fecha mínima a ingresar: {value}') {
        const call = (control) => {
            if (!control.value)
                return null;
            const value = new Date(control.value);
            value.setHours(0, 0, 0, 0);
            if (!(min instanceof Date))
                min = new Date(min);
            min.setHours(0, 0, 0, 0);
            if (value >= min) {
                return null;
            }
            else {
                let dateString = ServiwUtil.datePipe(min, format);
                return { other: error.replace('{value}', dateString) };
            }
        };
        return call;
    }
    /**
     * Properties serviw validator
     * @param objects
     * @param error
     * @returns
     */
    static properties(objects, error) {
        const call = (control) => {
            if (!control.value)
                return null;
            const num = Object.keys(control.value).length;
            if (objects < num)
                return null;
            else
                return { other: error };
        };
        return call;
    }
    static object(error) {
        const call = (control) => {
            if (!control.dirty)
                return null;
            const value = control.value;
            if (ServiwUtil.isObject(value))
                return null;
            else
                return { other: error };
        };
        return call;
    }
    /**
     * Desactivar controles de un formulario si esta asignado contenido
     * al primer ingreso
     */
    static disableEdit() {
        const call = (control) => {
            if (control && !control.dirty && (control.value != undefined || control.value != null))
                setTimeout(a => { control.disable(); }, 2);
            return null;
        };
        return call;
    }
    static nullValue() {
        const call = (control) => {
            if (control.value)
                return { notRequired: true };
            else
                return null;
        };
        return call;
    }
    /**
     * Consulta asíncrona, donde se valida un si existe un registro en el servidor.
     *
     * 1. Define el número de controles a suscribir
     * 2. Sección de retorno, Promise.
     *  2.1. Establecer control de validación al servidor
     *  2.2. Obtiene y asigna funciones de validación
     *
     * @param id identificador de cada valor si es objeto,
     * @param editable edita o no el con contenido
     * @requires Promise<any> in suscribeControls
     * @description se identifican propiedades, asignadas dentro de otra por un punto(.).
     * Se recomienda usar validación en los campos con @var ngIF ...control.errors para visializar errores si y solo si hay error
     */
    static validServer(id) {
        // 1.
        return null;
    }
    /**
     * Validar valores eliminando espacios en blanco
     */
    static blanks() {
        const call = (control) => {
            const value = control.value;
            if (!value || value.trim() == '')
                return { other: 'No se permiten espacios en blanco' };
            return null;
        };
        return call;
    }
    /**
     * Validar datos numericos, omitiendo valores nulos
     *
     * 1. Asignar valor ingresado, valida dato numerico
     * 2. Datos numericos invalidos
     *
     * @param element
     */
    static requiredNumber() {
        // 1.
        const call = (control) => {
            const value = control.value;
            const validNumber = isNaN(value);
            if (null != value && validNumber && !control.dirty)
                return { notNumber: true };
            return null;
        };
        return call;
    }
    /**
     * Establecer una lista personalizada de valores, que se validan
     * con el valor ingresado en el control.
     *
     * 1. Tipo de retorno, en AbstractControl
     *  1.1. Declara valor y valor valido.
     *  1.2. Define si un valor coincide con el actual
     *  1.3. Establece error.
     *
     * @param vals
     */
    static validData(values) {
        // 1.
        const call = (control) => {
            // 1.1.
            const value = control.value;
            let valid = false;
            if (control.dirty && control.valid) {
                // 1.2.
                for (let val of values)
                    if (ServiwUtil.objEquals(value, val)) {
                        valid = true;
                        break;
                    }
                // 1.3.
                if (!valid)
                    return { other: 'Valores Requeridos: ' + values.toString() };
            }
            return null;
        };
        return call;
    }
    static equals(value, error) {
        const call = (control) => {
            let valueC = control.value;
            if (valueC === value) {
                return null;
            }
            else {
                if (error)
                    error = error.replace('{value}', valueC);
                return { other: error || 'Valor unico a ingresar: ' + valueC };
            }
        };
        return call;
    }
    /**
     * Equals serviw validator
     * @param nameControl
     * @returns
     */
    static equalsIn(nameControl, error) {
        let start = false;
        const call = (control) => {
            const form = control.parent; // Only FormGroup is used in this API
            if (!form || (form && !form.dirty))
                return null;
            const controlIn = form.controls[nameControl]; // must be used ServiwControl
            if (controlIn.value) {
                controlIn.addUnqValidators(ServiwValidator.equals(control.value, error));
                controlIn.updateValueAndValidity({ emitEvent: false });
            }
            if (!start) {
                start = true;
                controlIn.valueChanges.subscribe(() => {
                    control.updateValueAndValidity();
                });
            }
            return ServiwValidator.equals(controlIn.value, error)(control);
        };
        return call;
    }
}

/**
 * Serviw control
 */
class ServiwControl extends FormControl {
    /**
     * Creates an instance of serviw control.
     * @param [formState]
     * @param [validatorOrOpts]
     * @param [asyncValidator]
     */
    constructor(formState = null, validatorOrOpts, asyncValidator) {
        super(formState, validatorOrOpts, asyncValidator);
        /**
         * Validators  of serviw control
         */
        this.validators = [];
        this._pristineValue = formState;
        const _validatorOrOpts = validatorOrOpts;
        this.setValidators(_validatorOrOpts);
    }
    /**
     * Gets pristine value
     */
    get pristineValue() { return this._pristineValue; }
    /**
     * Sets validators
     * @param val
     */
    setValidators(val) {
        this.validators = normalizeValidators(val);
        super.setValidators(this.validators);
    }
    /**
     * Adds unq validators
     * @param val
     */
    addUnqValidators(val) {
        const validators = [...this.validators, ...normalizeValidators(val)];
        super.setValidators(validators);
    }
    /**
     * Adds validators
     * @param val
     */
    addValidators(val) {
        this.setValidators(
        // merge current and added validators
        [...this.validators, ...normalizeValidators(val)]
            // remove duplicity
            .filter((v, i, a) => a.indexOf(v) === i));
    }
    /**
     * Marks as pristine
     * @param [opts]
     */
    markAsPristine(opts) {
        super.markAsPristine(opts);
        this._pristineValue = this.value;
    }
    ;
    /**
     * Sets value
     * @param value
     * @param options
     */
    setValue(value, options) {
        super.setValue(value, options);
        const byView = options && options.emitModelToViewChange === false;
        if (!byView)
            this._pristineValue = value;
    }
    ;
}
/**
 * Normalizes validators
 * @param val
 * @returns validators
 */
function normalizeValidators(val) {
    if (val === null)
        return [];
    if (Array.isArray(val))
        return val;
    return [val];
}

const DataApp = {
    storage: {},
    valid: () => !ServiwUtil.empty(DataApp.storage),
};

// -----------------------------------------------------
/**
 * Core property
 */
class CoreProperty {
    constructor(entity) {
        this.property = {};
        this.nameField = '';
        this.schema = '';
        this.loading = false;
        this.label = '';
        this.inputType = '';
        this.readonly = false;
        this.options = {};
        if (entity.json && !entity.label) {
            const jsonStorage = DataApp.storage[entity.json];
            if (!jsonStorage)
                throw new Error('No se ha cargado el json: ' + entity.json + ' en el jsonStorage');
            entity.label = jsonStorage.titleForm;
        }
    }
}
/**
 * Svw property graphql
 */
class CorePropertyGraphql extends CoreProperty {
    constructor(nameField, property) {
        super(property);
        this.nameField = nameField;
        this.property = property;
        const json = property.json;
        if (json && !DataApp.storage[json]['singular'])
            DataApp.storage[json]['singular'] = nameField;
        this.schema = SCHEMA_GRAPHQL;
    }
}
/**
 * Svw property rest
 */
class CorePropertyRest extends CoreProperty {
    constructor(nameField, property) {
        super(property);
        this.nameField = nameField;
        this.property = property;
        this.schema = SCHEMA_REST;
    }
}

/**
 * Serviw core json
 */
class ServiwCoreJson {
    /**
     * Creates an instance of serviw core json.
     */
    constructor(json) {
        this.json = json;
        this.validJson = !ServiwUtil.empty(json);
        this._id = undefined;
        this._coreFields = [];
        this._coreFieldsForm = [];
        if (!json.operations)
            json.operations = { read: true, create: true, update: true };
        if (!json.operations.read)
            json.operations.read = true;
        if (!json.buttons)
            json.buttons = { create: 'Nuevo', update: 'Editar', delete: 'Eliminar', save: 'Guardar', cancelSave: 'Cancelar' };
        if (!json.messages)
            json.messages = {};
        this.resetParams();
    }
    /** Emptys id */
    emptyId(data) {
        const id = this.id;
        if (!id)
            return true;
        if (data)
            return ServiwUtil.empty(data[id.name]);
        else
            return ServiwUtil.empty(id.value);
    }
    /** Gets id name */
    get idName() {
        return (this._id ? this._id.name : this.id.name) ?? '';
    }
    /** Gets id */
    get id() {
        // 1.
        if (this._id)
            return this._id;
        // 2.
        this.mapFields((f) => {
            if (f.property.id) {
                this._id = { name: f.nameField, field: f, value: f.property.value };
                return;
            }
        });
        // 3.
        if (null == this._id)
            throw new Error('Not found id: ' + this.json.titleForm);
        return this._id;
    }
    /** Maps fields */
    mapFields(call) {
        for (const key in this.json.fields) {
            // 1.
            const field = this.json.fields[key];
            field.templatesTable = field.templatesTable || {};
            field.templatesForm = field.templatesForm || {};
            // 2.
            if (this.schema == SCHEMA_REST)
                call(new CorePropertyRest(key, field));
            // 3.
            else if (this.schema == SCHEMA_GRAPHQL)
                call(new CorePropertyGraphql(key, field));
            else
                throw new Error('schema invalido');
        }
    }
    /** Gets core fields with template && displayColumn */
    get coreFields() {
        if (this._coreFields.length > 0)
            return this._coreFields;
        this.mapFields((f) => {
            if (!f.property.omitField) {
                if (f.property.json && !f.property.template)
                    f.property.template = MANY_TO_ONE;
                if (f.property.template)
                    this._coreFields.push({ name: f.nameField, property: f });
            }
        });
        return this._coreFields;
    }
    /** Cores fields form  */
    get coreFieldsForm() {
        if (this._coreFieldsForm.length > 0)
            return this._coreFieldsForm;
        this.mapFields((f) => {
            if (f.property.showForm != false || f.property.showDynamic) {
                f.property.type = f.property.type || TYPES.string;
                if (f.property.json && !f.property.template)
                    if (f.property.relation != ONE_TO_ONE)
                        f.property.template = f.property.relation;
                    else
                        f.property.template = MANY_TO_ONE;
                if (f.property.template)
                    this._coreFieldsForm.push({ name: f.nameField, property: f });
            }
        });
        return this._coreFieldsForm;
    }
    /** Values properties */
    static valueProperties(item, column, property) {
        if (!column || !item)
            return '';
        let dataView = null;
        if (property && property.json)
            dataView = column + '.' + DataApp.storage[property.json].displayField;
        const properties = dataView || column;
        if (!properties)
            return '';
        return ServiwUtil.valueProperties(item, properties);
    }
    /** Sets value id */
    setValueId(value) {
        this.id.value = value[this.idName];
    }
    /** Resets value id */
    resetValueId() {
        this.id.value = null;
    }
}

/**
 * Entity graphql
 * @template I Entity interface
 */
class EntityGraphql extends ServiwCoreJson {
    /**
     * Creates an instance of entity graphql.
     */
    constructor(json) {
        super(json);
        this.json = json;
        /** Schema  of entity graphql */
        this.schema = SCHEMA_GRAPHQL;
        /** Params  of entity graphql */
        this.params = {};
    }
    /**
     * Gets fields
     */
    get fields() {
        return this.json.fields;
    }
    /**
     * Resets params
     */
    resetParams() {
        this.params = this.paramsDefault();
    }
    /**
     * Params default
     * @returns
     */
    paramsDefault() {
        return EntityGraphql.paramsDefault(this.json.first);
    }
    /**
     * Params default
     * @param [first]
     * @param [after]
     * @returns
     */
    static paramsDefault(first, after) {
        return { first: { type: 'int', value: first, validations: { nullable: true } }, after: { type: 'String', value: after, validations: { nullable: true } } };
    }
    /**
     * Refactors query edges
     * @param fields
     * @returns query edges
     */
    fieldsQuery() {
        let data = this.idName + ',';
        this.mapFields((f) => {
            if (f.property.omitField)
                return;
            data += f.nameField + ',';
            const json = DataApp.storage[f.property.json];
            if (json) {
                const c = new EntityGraphql(json);
                if (f.property.relation == MANY_TO_MANY)
                    data += '{\n totalCount \n}';
                else if (f.schema == this.schema)
                    data += '{\n' + c.idName + ' ' + (json.displayField || '') + '' + (f.property.propertiesGet || '') + '\n},';
            }
        });
        return data;
    }
    /**
     * Gets fields
     * @param filters
     * @returns
     */
    getFields(filters) {
        let value = '';
        const array = filters.split(',');
        for (const field of array) {
            value += field.split(':')[0].trim() + '\n';
        }
        return value;
    }
}

/**
 * Entity rest
 */
class EntityRest extends ServiwCoreJson {
    /**
     * Creates an instance of entity rest.
     * @param json
     */
    constructor(json) {
        super(json);
        this.json = json;
        /**
         * Schema  of entity rest
         */
        this.schema = SCHEMA_REST;
        /**
         * Params  of entity rest
         */
        this.params = {};
    }
    /**
     * Gets fields
     */
    get fields() {
        return this.json.fields;
    }
    /**
     * Resets params
     */
    resetParams() {
        this.params = this.paramsDefault();
    }
    /**
     * Params default
     * @returns
     */
    paramsDefault() {
        return EntityRest.paramsDefault(this.json.first, this.json.filters);
    }
    /**
     * Params default
     * @param [first]
     * @param [filters]
     * @returns
     */
    static paramsDefault(first, filters) {
        return {
            first,
            filters: { ...filters }
        };
    }
    /**
     * Fields query
     */
    fieldsQuery() {
        return '';
    }
    /**
     * Gets fields
     * @param filters
     * @returns
     */
    getFields(filters) {
        return '';
    }
}

class ServiwBuilder extends FormBuilder {
    /**
     * Params serviw builder
     * @param controlsConfig
     * @param [extra]
     * @description [any => value, ValidatorFn => validations, AsyncValidatorFn => validations async]
     * @description Types => [] | [any] | [any, ValidatorFn] | [any, ValidatorFn, AsyncValidatorFn]
     * @returns group FormGroup
     */
    group(controlsConfig, extra) {
        return super.group(controlsConfig, extra);
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwBuilder.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwBuilder, deps: null, target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwBuilder.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwBuilder, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwBuilder, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });
/**
 * Serviw form
 * I -> interface
 */
class ServiwForm extends FormGroup {
    constructor(controls, entity, subscription) {
        super(controls);
        /**
         * Entity  of serviw form
         */
        this.entity = {};
        this.entity = entity ?? {};
        this.subscription = subscription || new Subscription();
    }
    /**
     * Unsubscribe this.subscription form
     */
    unsubscribeControls() {
        this.subscription.unsubscribe();
    }
}
/**
 * Serviw abstract control
 */
class ServiwFormControl extends FormControl {
    constructor() {
        super(...arguments);
        this.errors = null;
    }
}

class ServiwFormService {
    /**
     * Creates an instance of serviw form service.
     * @param builder
     */
    constructor(builder, settings) {
        this.builder = builder;
        this.settings = settings;
    }
    /**
     *
     * @param type
     */
    static validType(type) {
        if (!type)
            return '';
        const types = ['string', 'int', 'float', 'date', 'double', 'long', 'boolean', 'password'];
        return types.find((name) => { return name == type.toLocaleLowerCase(); });
    }
    /**
     * Refactors type
     * @param type
     * @param [forceType]
     * @returns
     */
    static refactorType(field, property) {
        let type = (property.json ? DataApp.storage[property.json]['singular'] || field : property.type) || field;
        if (property.relation == 'manyToMany')
            type = '[String]';
        if (property.forceType)
            return type;
        let _type = this.validType(type);
        if (!_type) {
            const index = type.indexOf('[');
            if (index >= 0)
                _type = type;
            else
                _type = 'string';
        }
        return ServiwUtil.firstCapitalLetter(_type);
    }
    /**
     * Refactors set data form
     * @param value
     * @param properties
     * @param [formGroup] set values null in fields not in value
     * @returns set data form
     */
    static refactorSetDataForm(value, properties, formGroup) {
        const keys = formGroup ? Object.keys(formGroup.value) : [];
        for (const field in properties) {
            const p = properties[field];
            if (p.template == 'date' && value[field])
                value[field] = new Date(value[field]);
            else if (p.template == 'checkbox' && p.showForm != false)
                value[field] = value[field] || false;
            else if (value[field] == undefined)
                value[field] = null;
            const index = keys.indexOf(field);
            if (index >= 0)
                keys.splice(index, 1);
        }
        for (const field of keys)
            value[field] = null;
        return value;
    }
    /**
     * Gets form json
     * @param json
     * @param [value]
     * @returns form json
     */
    getFormJson(json, value) {
        const entity = this.settings.scheme == SCHEMA_GRAPHQL ? new EntityGraphql(json) : new EntityRest(json);
        const form = this.getForm(entity);
        if (value)
            form.setValue(ServiwFormService.refactorSetDataForm(value, json.fields, form));
        return { form, entity };
    }
    /**
     * Gets form
     * @param json
     * @param [subscribe]
     * @returns form
     */
    getForm(json, subscribe = false) {
        const fields = json.fields;
        const serviwForm = new ServiwForm({}, json);
        for (const name in fields) {
            const property = this.configProperty(fields[name], name);
            const control = this.validation(property);
            serviwForm.addControl(name, control);
        }
        if (json.schema == SCHEMA_GRAPHQL)
            serviwForm.addControl('_id', new FormControl());
        if (subscribe) {
            const subscription = this.suscribeControls(serviwForm);
            serviwForm.subscription = subscription;
        }
        return serviwForm;
    }
    /**
     * Configs property
     * @param property
     * @param name
     * @returns
     */
    configProperty(property, name) {
        const showForm = true == property.showForm || undefined == property.showForm || property.showDynamic;
        if (!showForm)
            return property;
        if (property.template != 'select' && !property.inputType)
            property.inputType = property.template;
        if (ServiwFormService.validType(property.template)) //|| !property.filters)
            return property;
        if (!property.options)
            property.options = [{ label: 'Seleccionar', value: '' }];
        return property;
    }
    /**
     * Validations serviw form service
     * @param property
     * @returns validation
     */
    validation(property) {
        const showForm = true == property.showForm || undefined == property.showForm || property.showDynamic;
        if (!showForm)
            return this.builder.control(property.value);
        if (property.template == 'checkbox' && property.showForm != false)
            property.value = false;
        const control = new ServiwControl(property.value, this.validatorOrOpts(property));
        if (property.disable)
            control.disable();
        return control;
    }
    /**
     * Set validator for control
     * @param property
     */
    validatorOrOpts(property) {
        // 1.
        if (!property.validations)
            property.validations = {};
        if (property.relation == MANY_TO_MANY || property.template == MANY_TO_MANY)
            property.validations.nullable = true;
        // 2.
        const messages = property.validations.errors || {};
        const validObject = property.template == 'autocomplete' && property.options && property.options.length <= 1;
        const validations = property.validations;
        // 3.
        let validators = [
            !validations.nullable ? Validators.required : null,
            !ServiwUtil.isEmpty(validations.max) ? Validators.max(validations.max) : null,
            !ServiwUtil.isEmpty(validations.min) ? Validators.min(validations.min) : null,
            !ServiwUtil.isEmpty(validations.maxLength) ? Validators.maxLength(validations.maxLength) : null,
            !ServiwUtil.isEmpty(validations.minLength) ? Validators.minLength(validations.minLength) : null,
            !ServiwUtil.isEmpty(validations.pattern) ? Validators.pattern(validations.pattern) : null,
            !ServiwUtil.isEmpty(validations.minIn) ? (property.template == 'date' ? ServiwValidator.minDateIn(validations.minIn) : ServiwValidator.minIn(validations.minIn)) : null,
            !ServiwUtil.isEmpty(validations.maxIn) ? (property.template == 'date' ? ServiwValidator.maxDateIn(validations.maxIn) : ServiwValidator.maxIn(validations.maxIn)) : null,
            validations.minDate ? ServiwValidator.minDate(validations.minDate) : null,
            validations.maxDate ? ServiwValidator.maxDate(validations.maxDate) : null,
            validations.equalsIn ? ServiwValidator.equalsIn(validations.equalsIn, messages.equalsIn) : null,
            validations.email ? Validators.email : null,
            validObject ? ServiwValidator.object(property.label + ': es requerido/a') : null,
        ];
        // 4.
        validators = validators.filter(function (el) { return el != null; });
        return Validators.compose(validators);
    }
    validations() {
    }
    /**
     * Suscribir controles al cambio de valores, asignando el detalle de cada error
     *
     * 1. Selecciona los controles actuales
     * 2. Callback ejecutado al cambio de valores
     * 3. Validar cada control
     *
     * @param formGroup Formulario a suscribir
     * @description Se asigna como primer campo, nombre de la
     * variable en el servicio Links, y una ruta consecutiva, como opción
     * que se agrega a la ruta definida.
     *
     * Se reconoce el tipo de petición count o get, según la ultima palabra del link
     *
     * @param formGroup
     * @param [call]
     * @returns subscription
     * @example { CostoUnitarioPeriodoFormComponent#loadForm() }
     */
    suscribeControls(formGroup, call) {
        // 1.
        const controles = formGroup.controls;
        // 2.
        const callLocal = (next) => {
            if (formGroup.dirty) {
                // 2.1.
                for (let control in controles) {
                    let controlLocal = controles[control];
                    if (controlLocal.errors && controlLocal.dirty)
                        controlLocal = this.setErrors(controlLocal, control);
                }
                if (call)
                    call(next);
            }
        };
        // 3.
        const subscription = formGroup.valueChanges.subscribe(callLocal);
        return subscription;
    }
    /**
     * Suscribes control
     * @param control
     * @param [name]
     * @returns  control
     */
    suscribeControl(control, name = 'Control') {
        control.statusChanges.subscribe((a) => {
            if (control.invalid)
                this.setErrors(control, name);
        });
        return control;
    }
    /**
     * Establecer mensajes personalizados a errores establecidos en el formulario
     *
     * 1. Selecciona errores de un control
     * 2. Establece nombre del control
     * 3. Asignar detalle de cada error
     *
     * @param control control a validar errores
     * @param nameControl nombre del control
     */
    setErrors(control, nameControl, error) {
        // 1.
        if (error)
            control.setErrors(error);
        let errors = control.errors;
        if (!errors)
            return control;
        // 2.
        errors['summary'] = nameControl;
        // 3.
        if (errors['required'])
            errors['detail'] = "Campo requerido";
        else if (errors['notRequired'])
            errors['detail'] = "Campo debe ser nulo";
        else if (errors['maxlength'])
            errors['detail'] = "Máximo " + errors['maxlength'].requiredLength + " caracteres";
        else if (errors['minlength'])
            errors['detail'] = "Mínimo " + errors['minlength'].requiredLength + " caracteres";
        else if (errors['max'])
            errors['detail'] = "Valor máximo a ingresar: " + errors['max'].max;
        else if (errors['min'])
            errors['detail'] = "Ingrese un valor mayor ó igual a: " + errors['min'].min;
        else if (errors['exists'])
            errors['detail'] = "Ya existe un registro en el servidor";
        else if (errors['errorExists'])
            errors['detail'] = "Problema al validar en el servidor";
        else if (errors['pattern'])
            errors['detail'] = "El formato del campo no es válido";
        else if (errors['email'])
            errors['detail'] = "El formato del correo no es válido";
        else if (errors['other'])
            errors['detail'] = errors['other'];
        return control;
    }
    /**
     * Refactors data form
     * @param value
     * @param entity
     * @returns data form
     */
    static refactorDataForm(value, entity) {
        // 1.
        if (entity.schema == SCHEMA_REST)
            return value;
        const properties = entity.fields;
        const data = {};
        for (const name in value) {
            // 2.
            if (name == '_id')
                continue;
            const valF = value[name];
            if (valF instanceof Date || valF instanceof File) {
                data[name] = valF;
                continue;
            }
            else if (Array.isArray(valF) && properties[name].json)
                data[name] = this.refactorArrayValue(valF, new EntityGraphql(DataApp.storage[properties[name].json]));
            else if (properties[name].json && ServiwUtil.isObject(valF)) {
                const c = new EntityGraphql(DataApp.storage[properties[name].json]);
                data[name] = valF[c.idName];
            }
            else if (valF != undefined || valF != null)
                data[name] = valF;
        }
        return data;
    }
    /**
     * Refactors array value
     * @param array
     * @param entity
     * @returns
     */
    static refactorArrayValue(array, entity) {
        const data = [];
        for (const obj of array)
            data.push(obj[entity.idName]);
        return data;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwFormService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwFormService, deps: [{ token: ServiwBuilder }, { token: SettingsServiwCore }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwFormService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwFormService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwFormService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: ServiwBuilder }, { type: SettingsServiwCore }]; } });

;
/** Error exception */
class ErrorException {
    constructor(message, cause = '', localizedMessage = '', supressed = undefined) {
        this.cause = cause;
        this.message = message;
        this.localizedMessage = localizedMessage;
        this.suppressed = supressed;
    }
}
ErrorException.ERROR_NETWORK = 'network_error';
ErrorException.ERROR_VALIDATION = 'validation_error';
ErrorException.ERROR_USER = 'user_error';
ErrorException.ERROR_VAR_INCOMPLETE = 'var_imcomplete_error';

const queryTokens = {
    separators: [
        ' ',
        ','
    ],
    separatorTokens: [
        '{',
        '}'
    ],
    openBrace: '{',
    closeBrace: '}'
};
class ServiwGraphqlService {
    /**
     * Creates an instance of serviw query service.
     * @param http
     * @param settings
     */
    constructor(serviwRest, settings) {
        this.serviwRest = serviwRest;
        this.settings = settings;
        this.inSave = false;
    }
    /**
     * Gets rest
     */
    get rest() {
        return this.serviwRest;
    }
    /**
     * Excludes fields graphql
     * @param field
     * @returns fields graphql
     */
    excludeFieldsGraphql(field) {
        field = field.trim();
        const vals = ['first', 'after', 'confirmPassword'];
        if (!this.inSave)
            vals.push('plainPassword');
        const value = vals.find((value) => { return value == field; });
        ;
        return ServiwUtil.empty(value);
    }
    /**
     * Querys save
     *
     * Define data query for schema
     *
     * 1. Define functions response
     * 2. Callback update Form
     * 3. Data for schema Graphql
     * 4. Data for schema REST-POST
     *
     * @param entity Entity
     * @param formGroup FormGroup
     * @returns EntitySave
     */
    querySave(entity, formGroup) {
        // 1.
        let functions = {};
        const empyId = entity.emptyId(formGroup.value);
        const operationName = (empyId ? 'create' : 'update') + entity.json.singular;
        // 2.
        const callUpdate = (graphqlBody) => {
            return (response) => {
                if (response.errors)
                    return null;
                return this.updateFormGroup(response, formGroup, entity.json, operationName);
            };
        };
        // 3.
        const callPost = (body) => {
            return (refactor) => {
                return this.serviwRest.post(this.settings.API_GRAPHQL, body, {}, refactor);
            };
        };
        // 4.
        const getErrors = (response) => { if (response.errors)
            return response.errors; };
        const getError = (response) => {
            if (response.errors)
                if (Array.isArray(response.errors))
                    return response.errors[0].message || response.errors[0].debugMessage;
            return JSON.stringify(response.errors);
        };
        // 5.
        const data = this.queryAddEditGraphql(entity, formGroup);
        functions = { data, updateForm: callUpdate(data), http: callPost(data), getErrors, getError };
        return functions;
    }
    /**
     * Querys graphql text
     * @param table
     * @param paramsTypesValues sintax nameParam:type:value
     */
    queryGraphqlSyntax(table, paramsTypesValues) {
        const data = this.getParamsFields(paramsTypesValues);
        return this.queryGraphql(table, data.params, data.fields);
    }
    /**
     * Gets params fields
     * @param paramsTypesValues
     * @returns
     */
    getParamsFields(paramsTypesValues) {
        let fields = '';
        const params = {};
        const syntaxParam = paramsTypesValues.split(',');
        for (let f of syntaxParam) {
            f = f.trim();
            const data = f.split(':');
            if (this.excludeFieldsGraphql(data[0]) && data[0].search('_') <= 0)
                fields += data[0] + ',';
            if (data.length <= 1)
                continue;
            const type = data.length > 1 ? data[1] : null;
            params[data[0]] = { type, value: (data.length > 2 ? data[2] : undefined) };
        }
        return { params, fields };
    }
    /**
     * Querys single graphql
     * @param entity name
     * @param fields sintax id,name,...
     * @param valueId value
     * @param [nameId] id
     * @returns  query | refactor | post
     */
    querySingleGraphql(entity, fields, valueId, nameId = 'id') {
        // 1.
        let query = 'query ([types]) {\n' + entity + '([params]) {\n [fields] \n }\n }';
        query = query.replace('[types]', ('$' + nameId + ': ID!'));
        query = query.replace('[params]', nameId + ': $' + nameId);
        // 2.
        let keys = fields.trim().split(',');
        keys = keys.map(item => item.trim());
        query = query.replace('[fields]', keys.join('\n'));
        // 3.
        const variables = { [nameId]: valueId };
        const graphqlBody = { query, variables };
        // 4.
        const refactorQuery = (response) => {
            if (response.data && response.data[entity])
                return response.data[entity];
            return ServiwUtil.emptyObject();
        };
        // 5.
        const post = () => { return this.serviwRest.post(this.settings.API_GRAPHQL, graphqlBody); };
        // 6.
        const refactor = { refactor: refactorQuery, http: post };
        return refactor;
    }
    /**
     * Querys delete graphql
     * @param entity
     * @param fields
     * @param valueId
     * @param [nameId]
     * @returns delete graphql
     */
    queryDeleteGraphql(entity, fields, valueId, nameId = 'id') {
        // 1.
        const operationName = 'delete' + ServiwUtil.firstCapitalLetter(entity);
        let query = 'mutation ' + operationName + '([types]) {' + operationName + '(input: { \n[params] }) { [{entitySave] \n [fields] [entitySave}] \n}\n}';
        const startInput = this.settings.entityFieldSave ? ServiwUtil.firstLowercaseLetter(entity) + '{' : '';
        const endInput = this.settings.entityFieldSave ? '}' : '';
        // 2.
        query = query.replace('[{entitySave]', startInput);
        query = query.replace('[entitySave}]', endInput);
        query = query.replace('[types]', ('$' + nameId + ': ID!'));
        query = query.replace('[params]', nameId + ': $' + nameId);
        query = query.replace('[fields]', fields.split(',').join('\n'));
        // 3.
        const variables = { [nameId]: valueId };
        const graphqlBody = { query, variables };
        // 4.
        const refactorQuery = (response) => {
            if (response.data && response.data[operationName])
                return response.data[operationName][entity];
            return {};
        };
        // 5.
        const post = () => { return this.serviwRest.post(this.settings.API_GRAPHQL, graphqlBody); };
        // 6.
        const refactor = { refactor: refactorQuery, http: post };
        return refactor;
    }
    /**
     * Separate keys for graphql queries
     * @param keys for separate
     * @returns keys separate in array
     */
    separateKeys(keys) {
        keys = keys.trim();
        let separate = [];
        let token = '';
        for (let i = 0; i < keys.length; i++) {
            let local = keys[i];
            let agregate = false;
            for (let k of queryTokens.separators) {
                if (local == k) {
                    if (token.length > 0) {
                        separate.push(token);
                        token = '';
                    }
                    agregate = true;
                    break;
                }
            }
            if (agregate) {
                continue;
            }
            for (let k of queryTokens.separatorTokens) {
                if (local == k) {
                    if (token.length > 0) {
                        separate.push(token);
                        token = '';
                    }
                    separate.push(local);
                    agregate = true;
                    break;
                }
            }
            if (agregate) {
                continue;
            }
            token += local;
        }
        return separate;
    }
    /**
     * Querys graphql
     *
     * 1. Name Entity
     * 2. Define schema Graphql types, params
     * 3. Define fields of query
     * 4. Set variables, create graphqlBody
     * 5. refactor data response
     * 6. refactor data response in options
     *
     * @param table name Entity
     * @param params data query
     * @param fields response
     * @param [paginator] paginator
     * @returns query | refactor | getOptions
     */
    queryGraphql(table, params, fields, paginator = '') {
        // 1.
        const entity = table; //.replace(/y$/, "ies");
        // 2.
        const jsonFields = params;
        let query = 'query ([types]) {\n' + entity + '([params]) {\n edges{\n node{\n [fields] }\n cursor }\n [paginator] \n }\n }';
        query = query.replace('[types]', this.getTypesGraphql(jsonFields, false));
        query = query.replace('[params]', this.getParamsGraphql(jsonFields, false));
        // 3.
        let keys = this.separateKeys(fields);
        keys = keys.map(item => item.trim());
        keys = keys.filter((key) => key != '');
        query = query.replace('[fields]', keys.join('\n'));
        query = query.replace('[paginator]', paginator);
        // 4.
        const variables = ServiwUtil.setKeyValue(params, 'value');
        const graphqlBody = { query, variables };
        // 5.
        const refactorQuery = (response) => {
            if (response.data && response.data[entity])
                return this.refactorEdges(response.data[entity], keys);
            return null;
        };
        // 6.
        const getOptions = (response, fieldLabel = keys[0]) => {
            const data = refactorQuery(response);
            const options = [{ label: 'Seleccionar', value: '' }];
            for (const item of data)
                options.push({ label: item[fieldLabel || keys[0]], value: item });
            return options;
        };
        const post = () => { return this.serviwRest.post(this.settings.API_GRAPHQL, graphqlBody); };
        const getError = (response) => {
            if (response.errors)
                if (Array.isArray(response.errors))
                    return response.errors[0].message || response.errors[0].debugMessage;
            return JSON.stringify(response.errors);
        };
        const refactor = { query: graphqlBody, refactor: refactorQuery, getOptions, http: post, getError };
        return refactor;
    }
    /**
     * Refactors edges->nodes
     *
     * 1. get edges
     * 2. Valid Array
     * 3. refactor, edge for node
     *
     * @param data response
     * @param keys name properties
     * @returns
     */
    refactorEdges(data, keys) {
        // 1.
        data = data || {};
        const refactor = [];
        const edges = data.edges;
        // 2.
        if (!Array.isArray(edges))
            return [];
        // 3.
        let conti = 0;
        for (const edge of edges) {
            const node = {};
            for (let key of keys) {
                if (key == queryTokens.openBrace) {
                    conti++;
                    continue;
                }
                else if (key == queryTokens.closeBrace) {
                    conti--;
                    continue;
                }
                if (conti > 0)
                    continue;
                key = key.trim();
                node[key] = edge.node[key];
            }
            refactor.push(node);
        }
        return refactor;
    }
    /**
     * Update formulario in the response the server
     * @param response data
     * @param formGroup setValue
     * @param json Entity
     * @param [operationName] acción realizada create<Table>, update<Table>
     * @returns
     */
    updateFormGroup(response, formGroup, json, operationName) {
        // 1.
        let properties = 'data.' + operationName;
        if (this.settings.entityFieldSave)
            properties += '.' + ServiwUtil.firstLowercaseLetter(json.singular);
        const data = ServiwUtil.valueProperties(response, properties);
        if (!ServiwUtil.empty(data))
            formGroup.setValue(ServiwFormService.refactorSetDataForm(data, json.fields, formGroup));
        else
            console.error('Error al actualiza formGroup: ' + operationName);
        return formGroup.value;
    }
    /**
     * Define the scheme to save or edit the data of an entity
     *
     * 1. Valid data id
     * 2. Deefine create or update
     * 3. Create GraphqlBody
     *
     * @param entity Entity
     * @param formGroup FormGroup
     * @returns graphqBody
     */
    queryAddEditGraphql(entity, formGroup) {
        // 1.
        const empyId = entity.emptyId(formGroup.value);
        // 2.
        const operationName = (empyId ? 'create' : 'update') + entity.json.singular;
        // 3.
        const query = this.querySaveGraphql(entity, operationName, empyId);
        const variables = ServiwFormService.refactorDataForm(formGroup.value, entity);
        const graphqBody = { query, variables };
        return graphqBody;
    }
    /**
     * Create queries to save or edit in the graphql schema
     *
     * 1. Valid save or edit
     * 2. Structure  of query
     * 3. Replace types, params, fields
     *
     * @param entity Entity
     * @param operationName action
     * @param [empyId] add or edit
     * @returns save graphql
     */
    querySaveGraphql(entity, operationName, empyId) {
        // 1.
        if (undefined == empyId)
            empyId = entity.emptyId();
        // 2.
        let data = 'mutation ' + operationName + '([types]) {' + operationName + '(input: { \n[params] }) { [{entitySave] \n [fields] [entitySave}] \n}\n}';
        // 3.
        const startInput = this.settings.entityFieldSave && entity.json.singular ? ServiwUtil.firstLowercaseLetter(entity.json.singular) + '{' : '';
        const endInput = this.settings.entityFieldSave ? '}' : '';
        data = data.replace('[{entitySave]', startInput);
        data = data.replace('[entitySave}]', endInput);
        this.inSave = true;
        if (!entity.fields)
            throw new ErrorException('No se definieron campos para la entidad', ErrorException.ERROR_USER);
        data = data.replace('[types]', this.getTypesGraphql(entity.fields, empyId));
        data = data.replace('[params]', this.getParamsGraphql(entity.fields, empyId));
        this.inSave = false;
        data = data.replace('[fields]', this.getFieldsGraphql(entity.fields));
        return data;
    }
    /**
     * Gets types graphql
     *
     * 1. Get fields EntityJSON
     * 2. Valid empyId
     * 3. Define name and type of properties
     * 4. Remove ', '
     *
     * @param entity Entity
     * @param [empyId] add or edit
     * @returns types graphql
     */
    getTypesGraphql(fields, empyId) {
        // 1.
        let types = '';
        // 2.
        const required = (property) => {
            if (property.validations && property.validations.nullable)
                return ', ';
            return '!, ';
        };
        // 3. 
        for (const field in fields) {
            if (this.skipWriteOnMutationInput(field, fields[field]))
                continue;
            if (fields[field].id) {
                if (!empyId)
                    types += '$' + field + ': ID!, ';
            }
            else {
                const p = fields[field];
                types += '$' + field + ': ' + ServiwFormService.refactorType(field, p) + required(p);
            }
        }
        // 4.
        types = types.slice(0, types.length - 2);
        return types;
    }
    /**
     * Gets params graphql
     *
     * 1. Get fields EntityJSON
     * 2. Valid empyId
     * 3. Define properties
     * 4. Remove ', '
     *
     * @param entity entity
     * @param [empyId]  add or edit
     * @returns params graphql
     */
    getParamsGraphql(fields, empyId) {
        // 1.
        let params = '';
        // 2.
        for (const field in fields) {
            if (this.skipWriteOnMutationInput(field, fields[field]))
                continue;
            if (fields[field].id === true) {
                params += 'clientMutationId: ' + '"' + field + '", ';
                if (empyId)
                    continue;
            }
            params += field + ': $' + field + ', ';
        }
        // 4.
        params = params.slice(0, params.length - 2);
        return params;
    }
    /**
     * Skips write on mutation input
     * @param field
     * @param property
     * @returns true if write on mutation input
     */
    skipWriteOnMutationInput(field, property) {
        return property.omitInput || this.inSave && !this.excludeFieldsGraphql(field);
    }
    /**
     * Gets fields graphql
     *
     * 1. fields storage
     * 2. Set fields for each property
     *
     * @param properties fields
     * @returns fields graphql
     */
    getFieldsGraphql(properties) {
        // 1.
        let data = '';
        for (const name in properties) {
            // 2.
            if (properties[name].omitField || !this.excludeFieldsGraphql(name))
                continue;
            const field = properties[name];
            const entity = field.json ? new EntityGraphql(DataApp.storage[field.json]) : null;
            data += name + ' ';
            const filters = [];
            let propertiesGet = field.propertiesGet ? field.propertiesGet.split(',') : [];
            propertiesGet.map((f) => { filters.push(f.trim()); });
            let type = entity ? DataApp.storage[field.json]['singular'] || name : field.type;
            switch (field.relation) {
                case MANY_TO_MANY:
                    type = `[${type}]`;
                    break;
                case ONE_TO_MANY:
                    type = `[${type}]`;
                    break;
            }
            if (type.indexOf('[') >= 0)
                //data += '{\n edges\n{\n node{\n' + json.idName + '\n' + filters.join('\n') + '\n}\n}\n}';
                data += '{ totalCount }';
            else if (entity)
                data += '{\n' + entity.idName + '\n' + filters.join('\n') + ' ' + (entity.json.displayField || '') + '\n}\n';
            else
                data += '\n';
        }
        return data;
    }
    /**
     * Gets field filter
     * @template S
     * @param property
     * @param value
     * @param [forceType]
     * @returns field filter
     */
    static getFieldFilter(property, value, forceType = false) {
        if (!forceType && Array.isArray(value))
            forceType = true;
        if (!property)
            property = { type: 'string', validations: { nullable: value == null } };
        const filter = {
            type: ServiwUtil.firstCapitalLetter(property.type),
            validations: { ...property.validations },
            value,
            forceType
        };
        return filter;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwGraphqlService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwGraphqlService, deps: [{ token: ServiwRest }, { token: SettingsServiwCore }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwGraphqlService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwGraphqlService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwGraphqlService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: ServiwRest }, { type: SettingsServiwCore }]; } });

class ServiwQueryService {
    /**
     * Creates an instance of serviw query service.
     * @param settings
     * @param serviwGraphql
     */
    constructor(settings, graphql) {
        this.settings = settings;
        this.graphql = graphql;
    }
    /**
     * Querys single
     * @param entity
     * @returns single
     */
    querySingle(entity) {
        // 1.
        const id = entity.id;
        if (ServiwUtil.empty(id.name) || ServiwUtil.empty(id.value)) {
            throw new ErrorException('Error al consultar registro de ' + entity.json.titleTable + ' ' + id, ErrorException.ERROR_VAR_INCOMPLETE);
        }
        // 2.
        let query;
        switch (entity.schema) {
            // 2.1
            case SCHEMA_GRAPHQL:
                const fields = this.graphql.getFieldsGraphql(entity.fields);
                const type = ServiwUtil.firstLowercaseLetter(entity.json.singular);
                query = this.graphql.querySingleGraphql(type, fields, id.value, id.name);
                break;
            // 2.2
            case SCHEMA_REST:
                const refactor = (response) => {
                    if (ServiwUtil.isObject(response) && response[id.name])
                        return response;
                    else
                        return {};
                };
                const get = () => {
                    const link = this.settings.API + entity.json.linkRest;
                    return this.graphql.rest.get(link, { [id.name]: id.value });
                };
                query = { refactor, http: get };
                break;
            default:
                throw new ErrorException('Esquema no soportado', ErrorException.ERROR_USER);
        }
        return query;
    }
    /**
     * Gets data
     * @param entity
     */
    getData(entity) {
        let query;
        switch (entity.schema) {
            // 2.
            case SCHEMA_GRAPHQL:
                if (!entity.json.plural && !entity.json.singular)
                    throw new ErrorException('No se definieron los datos de la entidad', ErrorException.ERROR_USER);
                const table = ServiwUtil.firstLowercaseLetter(entity.json.plural || entity.json.singular);
                const getFields = entity.fieldsQuery();
                const paginator = entity.params['after'] ? 'totalCount pageInfo { endCursor }' : '';
                query = this.graphql.queryGraphql(table, entity.params, getFields, paginator);
                break;
            // 3.
            case SCHEMA_REST:
                query = this.getQueryDataRest(entity);
                break;
        }
        return query;
    }
    /**
     * Delete item
     * @param entity
     * @returns query
     */
    deleteItem(entity, item) {
        let query;
        const valueId = item[entity.idName];
        switch (entity.schema) {
            // 2.
            case SCHEMA_GRAPHQL:
                const table = ServiwUtil.firstLowercaseLetter(entity.json.singular);
                const fields = entity.idName + ',' + (entity.json.displayField || '');
                query = this.graphql.queryDeleteGraphql(table, fields, valueId, entity.idName);
                break;
            // 3.
            case SCHEMA_REST:
                //
                let url = this.settings.API + entity.json.linkRest;
                const http = () => {
                    return this.graphql.rest.delete(url, { [entity.idName]: valueId });
                };
                const refactor = (response) => { return response; };
                query = { http, refactor };
                break;
        }
        return query;
    }
    /**
     * Gets query data rest
     * @param entity
     * @returns query data rest
     */
    getQueryDataRest(entity) {
        // 1.
        const refactor = (data) => {
            if (Array.isArray(data))
                return data;
            else if (ServiwUtil.isObject(data)) {
                for (let key in data)
                    if (Array.isArray(data[key]))
                        return data[key];
            }
            else
                return [];
        };
        // 2.
        const http = () => {
            let url = this.settings.API + entity.json.linkRest;
            if (entity.json.apiFilter)
                return this.graphql.rest.filter(url, entity.params);
            else {
                const params = this.graphql.rest.refactorFilters(entity.params);
                return this.graphql.rest.get(url, params);
            }
        };
        return { refactor, http };
    }
    /**
     * Querys save
     *
     * Define data query for schema
     *
     * 1. Define functions response
     * 2. Callback update Form
     * 3. Data for schema Graphql
     * 4. Data for schema REST-POST
     *
     * @param entity Entity
     * @param formGroup FormGroup
     * @returns EntitySave
     */
    querySave(entity, formGroup) {
        // 1.
        let query = {};
        switch (entity.schema) {
            case SCHEMA_GRAPHQL:
                query = this.graphql.querySave(entity, formGroup);
                break;
            case SCHEMA_REST:
                query = this.getQuerySaveRest(entity, formGroup);
                break;
        }
        return query;
    }
    /**
     * Gets query save rest
     * @returns query save rest
     */
    getQuerySaveRest(entity, formGroup) {
        const link = this.graphql.settings.API_GRAPHQL + entity.json.linkRest;
        let http;
        if (entity.emptyId(formGroup.value))
            http = () => { return this.graphql.rest.post(link, formGroup.value); };
        else
            http = () => { return this.graphql.rest.put(link, formGroup.value); };
        const updateForm = (response) => {
            if (ServiwUtil.isObject(response) && response[entity.idName])
                return response;
            return null;
        };
        const getErrors = (response) => {
            if (!ServiwUtil.isObject(response))
                return response;
            else if (!response[entity.idName])
                return response;
            return null;
        };
        return { http, updateForm, getErrors, data: formGroup.value };
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwQueryService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwQueryService, deps: [{ token: SettingsServiwCore }, { token: ServiwGraphqlService }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwQueryService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwQueryService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwQueryService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: SettingsServiwCore }, { type: ServiwGraphqlService }]; } });

var $this;
class ServiwFormField {
    /**
     * Creates an instance of serviw form field.
     */
    constructor() {
        this.__data = {};
        this.local = true;
        $this = this;
    }
    /**
     * Gets field
     */
    get property() {
        return this.field?.property;
    }
    /**
     * Gets field name
     */
    get fieldName() {
        return this.field ? this.field.nameField : '';
    }
    /**
     * Sets input
     */
    set formGroup(val) {
        if (!(val instanceof FormGroup))
            throw new Error('Tipo de dato no valido');
        this.__data.formGroup = val;
        if (this.local && this.field?.property.json) {
            this.control?.valueChanges.subscribe(() => { this.__data.view = undefined; });
        }
    }
    /**
     * Gets form group
     */
    get formGroup() {
        return this.__data.formGroup;
    }
    /**
     * Gets control
     */
    get control() {
        return this.__data.formGroup?.controls[this.fieldName];
    }
    /**
     * Gets class label
     */
    get classLabel() {
        const validations = this.property?.validations;
        if (validations && !validations.nullable)
            return 'required';
        return '';
    }
    /**
     * Gets placeholder label
     */
    get placeholderLabel() {
        let label = this.property?.label || (this.property?.json ? DataApp.storage[this.property.json] : this.fieldName);
        if (this.classLabel != '')
            label += ' * ';
        return label;
    }
    /**
     * Gets item display
     */
    get itemDisplay() {
        const label = this.property?.json ? DataApp.storage[this.property.json].displayField ?? '' : this.field?.nameField ?? '';
        return (value) => {
            if (typeof value == 'object' && undefined != value || null != value && label)
                return value[label];
            return value;
        };
    }
    /**
     * Comparar objetos por id, recomendado para template <select [compareWith]="compareFn"
     * @param item1
     * @param item2
     * @returns true if item1 == item2
     */
    compareFn(item1, item2) {
        const esquema = ServiwUtil.empty($this.id) ? null : { [$this.id]: '' };
        return ServiwUtil.objEquals(item1, item2, esquema);
    }
    /**
     * Inits on clicks
     */
    initOnClicks() {
        this.onClicks = new EventEmitter();
    }
    /**
     * Submits form
     */
    submitForm() {
        this.onClicks?.emit('submit');
    }
    /**
     * Gets data view
     */
    get dataView() {
        if (!this.__data.view) {
            const field = this.property?.json ? DataApp.storage[this.property.json].displayField : this.fieldName;
            if (!field) {
                throw 'Nombre del campo no definido o no se definió display field para ' + this.fieldName;
            }
            this.__data.view = ServiwCoreJson.valueProperties(this.control?.value, field);
        }
        return this.__data.view;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwFormField.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwFormField, deps: [], target: i0.ɵɵFactoryTarget.Directive });
/** @nocollapse */ /** @nocollapse */ ServiwFormField.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.1.2", type: ServiwFormField, selector: "[libServiwFormField]", inputs: { entity: "entity", field: "field", formGroup: "formGroup" }, outputs: { onClicks: "onClicks" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwFormField, decorators: [{
            type: Directive,
            args: [{
                    selector: '[libServiwFormField]'
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { entity: [{
                type: Input
            }], field: [{
                type: Input,
                args: ['field']
            }], onClicks: [{
                type: Output
            }], formGroup: [{
                type: Input
            }] } });

class ServiwFormDirective extends ServiwFormField {
    /**
     * Creates an instance of serviw form directive.
     * @param viewContainerRef
     * @param serviwQueryService
     * @param cd
     */
    constructor(viewContainerRef, serviwQueryService, cd, renderer2) {
        super();
        this.viewContainerRef = viewContainerRef;
        this.serviwQueryService = serviwQueryService;
        this.cd = cd;
        this.renderer2 = renderer2;
        this.local = false;
        this.initOnClicks();
        this.config = {};
    }
    /**
     * after content init
     *
     * @description Inicializar ajustes de cada plantilla (campos de formulario)
     */
    ngOnInit() {
        if (!this.field)
            return console.error('Error al cargal plantilla');
        this.config.showForm = this.property?.showForm;
        this.loadTemplate();
        this.loadCallBacks();
        // this.validOptions();
    }
    /**
     * Loads template
     * @returns true if template
     */
    loadTemplate() {
        if (!this.property?.showForm)
            return false;
        const settings = this.serviwQueryService.settings;
        const factory = settings.resolver.resolveComponentFactory(this.template);
        this.componentRef = this.viewContainerRef.createComponent(factory);
        this.renderer2.addClass(this.componentRef.location.nativeElement, 'serviw-form-field');
        this.renderer2.addClass(this.componentRef.location.nativeElement, this.fieldName);
        this.updateComponentRef();
        return true;
    }
    /**
     * Gets template
     */
    get template() {
        if (!this.property?.template) {
            throw new Error('La propiedad no se encuentra definida');
        }
        const formTemplate = this.serviwQueryService.settings.formTemplate[this.property.template];
        const htmlTemplate = formTemplate.template;
        if (htmlTemplate)
            return htmlTemplate;
        throw new Error('No se ha definido una platilla para el tipo: ' + this.property.type + ', Campo:  ' + this.property.label);
    }
    /**
     * Updates component ref
     * @param [changes]
     */
    updateComponentRef(changes) {
        if (!this.componentRef)
            throw new Error('No se definió la referencia del componente');
        this.componentRef.instance.field = this.field;
        this.componentRef.instance.formGroup = this.formGroup;
        this.componentRef.instance.entity = this.entity;
        this.componentRef.instance.onClicks = this.onClicks;
        if (changes)
            this.componentRef.hostView.detectChanges();
    }
    /**
     * Loads call backs
     */
    loadCallBacks() {
        const callToogle = (val) => {
            val = val == undefined ? this.property?.showForm : !val;
            if (!this.property)
                throw new Error('No se definió la propiedad de este componente');
            if (val != false && this.componentRef) {
                this.property.showForm = false;
                this.viewContainerRef.clear();
                this.componentRef.destroy();
            }
            else if (val == false && this.viewContainerRef.length == 0) {
                this.property.showForm = true;
                this.loadTemplate();
            }
        };
        if (!this.property || !this.entity?.fields)
            throw new Error('No se definió la propiedad de este componente o el campo');
        this.property.toogle = callToogle;
        this.entity.fields[this.fieldName] = this.property;
    }
    /**
     * Valids options
     * @deprecated
     */
    validOptions() {
        if (!this.property?.options)
            throw new Error('No se definió la propiedad');
        const emptyOptions = ServiwUtil.empty(this.property.options);
        if (emptyOptions || !emptyOptions && this.property.options.length <= 1)
            if (this.property.template == 'select')
                this.loadOptions();
            else if (this.property.template == 'autocomplete')
                this.loadAutocomplete();
    }
    /**
     * Loads autocomplete
     * @deprecated
     */
    loadAutocomplete() {
        if (!this.field?.nameField) {
            throw 'No se encuentra el nombre del campo in loadAutocomplete';
        }
        const control = this.formGroup.controls[this.field.nameField];
        // 2.
        const observer = (value) => {
            if (!this.formGroup.dirty || ServiwUtil.empty(value) || typeof value === "object")
                return;
            if (this.timeout)
                clearTimeout(this.timeout);
            this.timeout = setTimeout(() => { this.loadOptions(value); }, 600);
        };
        // 3.
        control.valueChanges.subscribe(observer);
    }
    /**
     * Loads options, consulta opciones para select del servidor
     * @deprecated
     * @returns options
     */
    loadOptions(value = '') {
        // 1.
        this.resetOptions = true;
        if (this.property)
            this.property.options = [{ label: 'Cargando', value: value }];
    }
    /**
     * on destroy
     */
    ngOnDestroy() {
        if (!this.property)
            return;
        if (this.resetOptions)
            this.property.options = [];
        this.property.showForm = this.config.showForm;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwFormDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwFormDirective, deps: [{ token: i0.ViewContainerRef }, { token: ServiwQueryService }, { token: i0.ChangeDetectorRef }, { token: i0.Renderer2 }], target: i0.ɵɵFactoryTarget.Directive });
/** @nocollapse */ /** @nocollapse */ ServiwFormDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.1.2", type: ServiwFormDirective, selector: "[libServiwForm]", usesInheritance: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwFormDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[libServiwForm]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ViewContainerRef }, { type: ServiwQueryService }, { type: i0.ChangeDetectorRef }, { type: i0.Renderer2 }]; } });

class ServiwCoreModule {
}
/** @nocollapse */ /** @nocollapse */ ServiwCoreModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ /** @nocollapse */ ServiwCoreModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreModule, declarations: [ServiwFormDirective], imports: [CommonModule], exports: [ServiwFormDirective] });
/** @nocollapse */ /** @nocollapse */ ServiwCoreModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreModule, providers: [
        ServiwFormService,
        ServiwUtil,
        ServiwRest,
        CommonModule
    ], imports: [[
            CommonModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule
                    ],
                    declarations: [
                        ServiwFormDirective,
                    ],
                    exports: [
                        ServiwFormDirective,
                    ],
                    providers: [
                        ServiwFormService,
                        ServiwUtil,
                        ServiwRest,
                        CommonModule
                    ],
                }]
        }] });

const INPUTNUMBER_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => InputNumber),
    multi: true
};
/**
 * Directive
 * @see [github] https://github.com/primefaces/primeng/blob/master/src/app/components/inputnumber/inputnumber.ts
 * @see [primeng] https://www.primefaces.org/primeng/showcase/#/inputnumber
 */
class InputNumber {
    constructor(el) {
        this.showButtons = false;
        this.format = true;
        this.buttonLayout = "stacked";
        this.useGrouping = true;
        this.incrementButtonIcon = 'pi pi-chevron-up';
        this.decrementButtonIcon = 'pi pi-chevron-down';
        this.mode = "decimal";
        this.step = 1;
        this.onFocus = new EventEmitter();
        this.onBlur = new EventEmitter();
        this.onModelChange = () => { };
        this.onModelTouched = () => { };
        this.input = el;
        el.nativeElement.oninput = this.onInput.bind(this);
        el.nativeElement.onkeydown = this.onInputKeyDown.bind(this);
        el.nativeElement.onkeypress = this.onInputKeyPress.bind(this);
        el.nativeElement.onpaste = this.onPaste.bind(this);
        el.nativeElement.onclick = this.onInputClick.bind(this);
        el.nativeElement.onfocus = this.onInputFocus.bind(this);
        el.nativeElement.onblur = this.onInputBlur.bind(this);
    }
    ngOnInit() {
        this.numberFormat = new Intl.NumberFormat(this.locale, this.getOptions());
        const numerals = [...new Intl.NumberFormat(this.locale, { useGrouping: false }).format(9876543210)].reverse();
        const index = new Map(numerals.map((d, i) => [d, i]));
        this._numeral = new RegExp(`[${numerals.join('')}]`, 'g');
        this._decimal = this.getDecimalExpression();
        this._group = this.getGroupingExpression();
        this._minusSign = this.getMinusSignExpression();
        this._currency = this.getCurrencyExpression();
        this._suffix = new RegExp(`[${this.suffix || ''}]`, 'g');
        this._prefix = new RegExp(`[${this.prefix || ''}]`, 'g');
        this._index = (d) => index.get(d);
    }
    formatValue(value) {
        if (value != null && value != undefined) {
            if (this.format) {
                let formatter = new Intl.NumberFormat(this.locale, this.getOptions());
                let formattedValue = formatter.format(value);
                if (this.prefix) {
                    formattedValue = this.prefix + formattedValue;
                }
                if (this.suffix) {
                    formattedValue = formattedValue + this.suffix;
                }
                return formattedValue;
            }
            return value;
        }
        return '';
    }
    formattedValue() {
        return this.formatValue(this.value ?? 0);
    }
    onInput(event) {
        if (this.isSpecialChar) {
            event.target.value = this.lastValue;
        }
        this.isSpecialChar = false;
    }
    onInputKeyDown(event) {
        this.lastValue = event.target.value;
        if (event.shiftKey || event.altKey) {
            this.isSpecialChar = true;
            return;
        }
        let selectionStart = event.target.selectionStart;
        let selectionEnd = event.target.selectionEnd;
        let inputValue = event.target.value;
        if (event.altKey) {
            event.preventDefault();
        }
        switch (event.which) {
            //up
            case 38:
                this.spin(event, 1);
                event.preventDefault();
                break;
            //down
            case 40:
                this.spin(event, -1);
                event.preventDefault();
                break;
            //left
            case 37:
                if (!this.isNumeralChar(inputValue.charAt(selectionStart - 1))) {
                    event.preventDefault();
                }
                break;
            //right
            case 39:
                if (!this.isNumeralChar(inputValue.charAt(selectionStart))) {
                    event.preventDefault();
                }
                break;
            //backspace
            case 8: {
                event.preventDefault();
                let newValueStr = null;
                if (selectionStart === selectionEnd) {
                    let deleteChar = inputValue.charAt(selectionStart - 1);
                    let decimalCharIndex = inputValue.search(this._decimal);
                    this._decimal.lastIndex = 0;
                    if (this.isNumeralChar(deleteChar)) {
                        if (this._group.test(deleteChar)) {
                            this._group.lastIndex = 0;
                            newValueStr = inputValue.slice(0, selectionStart - 2) + inputValue.slice(selectionStart - 1);
                        }
                        else if (this._decimal.test(deleteChar)) {
                            this._decimal.lastIndex = 0;
                            this.input.nativeElement.setSelectionRange(selectionStart - 1, selectionStart - 1);
                        }
                        else if (decimalCharIndex > 0 && selectionStart > decimalCharIndex) {
                            newValueStr = inputValue.slice(0, selectionStart - 1) + '0' + inputValue.slice(selectionStart);
                        }
                        else {
                            newValueStr = inputValue.slice(0, selectionStart - 1) + inputValue.slice(selectionStart);
                        }
                    }
                    if (newValueStr != null) {
                        this.updateValue(event, newValueStr, 'delete-single');
                    }
                }
                else {
                    newValueStr = this.deleteRange(inputValue, selectionStart, selectionEnd);
                    this.updateValue(event, newValueStr, 'delete-range');
                }
                break;
            }
            default:
                break;
        }
    }
    onInputKeyPress(event) {
        event.preventDefault();
        let code = event.which || event.keyCode;
        let char = String.fromCharCode(code);
        if ((48 <= code && code <= 57) || this.isMinusSign(char)) {
            this.insert(event, char);
        }
    }
    onPaste(event) {
        event.preventDefault();
        let data = (event.clipboardData || window['clipboardData']).getData('Text');
        if (data) {
            let filteredData = this.parseValue(data);
            if (filteredData != null) {
                this.insert(event, filteredData.toString());
            }
        }
    }
    onInputClick() {
        this.initCursor();
    }
    onInputFocus(event) {
        this.focused = true;
        this.onFocus.emit(event);
    }
    onInputBlur(event) {
        this.focused = false;
        this.onBlur.emit(event);
    }
    onUpButtonMouseDown(event) {
        this.input.nativeElement.focus();
        this.repeat(event, null, 1);
        event.preventDefault();
    }
    onUpButtonMouseUp() {
        this.clearTimer();
    }
    onUpButtonMouseLeave() {
        this.clearTimer();
    }
    onUpButtonKeyDown(event) {
        if (event.keyCode === 32 || event.keyCode === 13) {
            this.repeat(event, null, 1);
        }
    }
    onUpButtonKeyUp() {
        this.clearTimer();
    }
    onDownButtonMouseDown(event) {
        this.input.nativeElement.focus();
        this.repeat(event, null, -1);
        event.preventDefault();
    }
    onDownButtonMouseUp() {
        this.clearTimer();
    }
    onDownButtonMouseLeave() {
        this.clearTimer();
    }
    onDownButtonKeyUp() {
        this.clearTimer();
    }
    onDownButtonKeyDown(event) {
        if (event.keyCode === 32 || event.keyCode === 13) {
            this.repeat(event, null, -1);
        }
    }
    spin(event, dir) {
        let step = this.step * dir;
        let currentValue = this.value || 0;
        let newValue = currentValue + step;
        if (this.min !== null && this.min != undefined && newValue < this.min) {
            newValue = this.min;
        }
        if (this.max !== null && this.max != undefined && newValue > this.max) {
            newValue = this.max;
        }
        this.updateInput(newValue + '', 'spin');
        this.updateModel(event, newValue);
    }
    repeat(event, interval, dir) {
        let i = interval || 500;
        this.clearTimer();
        this.timer = setTimeout(() => {
            this.repeat(event, 40, dir);
        }, i);
        this.spin(event, dir);
    }
    clearTimer() {
        if (this.timer) {
            clearInterval(this.timer);
        }
    }
    insert(event, text) {
        let selectionStart = this.input.nativeElement.selectionStart;
        let selectionEnd = this.input.nativeElement.selectionEnd;
        let inputValue = this.input.nativeElement.value.trim();
        let maxFractionDigits = this.numberFormat.resolvedOptions().maximumFractionDigits;
        let newValueStr;
        let decimalCharIndex = inputValue.search(this._decimal);
        this._decimal.lastIndex = 0;
        if (decimalCharIndex > 0 && selectionStart > decimalCharIndex) {
            if ((selectionStart + text.length - (decimalCharIndex + 1)) <= maxFractionDigits) {
                newValueStr = inputValue.slice(0, selectionStart) + text + inputValue.slice(selectionStart + text.length);
                this.updateValue(event, newValueStr, 'insert');
            }
        }
        else {
            newValueStr = this.insertText(inputValue, text, selectionStart, selectionEnd);
            this.updateValue(event, newValueStr, 'insert');
        }
    }
    /**
 * Precisions value
 * @param newValueStr
 * @returns value
 */
    precisionValue(newValueStr) {
        if (newValueStr && this.mode == 'decimal') {
            newValueStr += '';
            const max = parseFloat(newValueStr.split('.')[0]);
            if (this.precision && max > parseFloat(this.precision))
                return newValueStr.slice(0, this.precision.length);
        }
        return newValueStr;
    }
    insertText(value, text, start, end) {
        let newValueStr;
        if ((end - start) === value.length)
            newValueStr = text;
        else if (start === 0)
            newValueStr = text + value.slice(end);
        else if (end === value.length)
            newValueStr = value.slice(0, start) + text;
        else
            newValueStr = value.slice(0, start) + text + value.slice(end);
        return newValueStr;
    }
    initCursor() {
        let selectionStart = this.input.nativeElement.selectionStart;
        let inputValue = this.input.nativeElement.value;
        let valueLength = inputValue.length;
        let index = null;
        let char = inputValue.charAt(selectionStart);
        if (this.isNumeralChar(char)) {
            return;
        }
        //left
        let i = selectionStart - 1;
        while (i >= 0) {
            char = inputValue.charAt(i);
            if (this.isNumeralChar(char)) {
                index = i;
                break;
            }
            else {
                i--;
            }
        }
        if (index !== null) {
            this.input.nativeElement.setSelectionRange(index + 1, index + 1);
        }
        else {
            i = selectionStart + 1;
            while (i < valueLength) {
                char = inputValue.charAt(i);
                if (this.isNumeralChar(char)) {
                    index = i;
                    break;
                }
                else {
                    i++;
                }
            }
            if (index !== null) {
                this.input.nativeElement.setSelectionRange(index, index);
            }
        }
    }
    updateInput(value, operation) {
        let currentLength = this.input.nativeElement.value.length;
        if (this.precision) {
            value = this.precisionValue(value);
            if (value.length == this.precision.length)
                this.input.nativeElement.selectionStart = this.precision.length;
        }
        if (currentLength === 0) {
            this.input.nativeElement.value = this.formatValue(+value);
            this.input.nativeElement.setSelectionRange(0, 0);
            this.initCursor();
            this.input.nativeElement.setSelectionRange(this.input.nativeElement.selectionStart + 1, this.input.nativeElement.selectionStart + 1);
        }
        else {
            let selectionStart = this.input.nativeElement.selectionEnd;
            let selectionEnd = this.input.nativeElement.selectionEnd;
            this.input.nativeElement.value = this.formatValue(+value);
            let newLength = this.input.nativeElement.value.length;
            if (newLength === currentLength) {
                if (operation === 'insert')
                    this.input.nativeElement.setSelectionRange(selectionEnd + 1, selectionEnd + 1);
                else if (operation === 'delete-single')
                    this.input.nativeElement.setSelectionRange(selectionEnd - 1, selectionEnd - 1);
                else if (operation === 'delete-range')
                    this.input.nativeElement.setSelectionRange(selectionStart, selectionStart);
                else if (operation === 'spin')
                    this.input.nativeElement.setSelectionRange(selectionStart, selectionEnd);
            }
            else {
                selectionEnd = selectionEnd + (newLength - currentLength);
                this.input.nativeElement.setSelectionRange(selectionEnd, selectionEnd);
            }
        }
        this.input.nativeElement.setAttribute('aria-valuenow', value);
    }
    updateModel(event, value) {
        this.value = value;
        this.onModelChange(value);
    }
    updateValue(event, valueStr, operation) {
        if (valueStr) {
            let newValue = this.parseValue(valueStr);
            if (undefined === newValue)
                return;
            let valid = this.isWithinRange(newValue);
            if (valid) {
                this.updateInput(newValue + '', operation);
                this.updateModel(event, newValue);
            }
        }
    }
    deleteRange(value, start, end) {
        let newValueStr;
        if ((end - start) === value.length)
            newValueStr = '';
        else if (start === 0)
            newValueStr = value.slice(end);
        else if (end === value.length)
            newValueStr = value.slice(0, start);
        else
            newValueStr = value.slice(0, start) + value.slice(end);
        return newValueStr;
    }
    isWithinRange(value) {
        return value == null || ((this.min == null || value > this.min) && (this.max == null || value < this.max));
    }
    isNumeralChar(char) {
        if (char.length === 1 && (this._numeral.test(char) || this._decimal.test(char) || this._group.test(char) || this._minusSign.test(char))) {
            this.resetRegex();
            return true;
        }
        return false;
    }
    isMinusSign(char) {
        if (this._minusSign.test(char)) {
            this._minusSign.lastIndex = 0;
            return true;
        }
        return false;
    }
    parseValue(text) {
        let filteredText = text.trim()
            .replace(/\s/g, '')
            .replace(this._currency, '')
            .replace(this._group, '')
            .replace(this._suffix, '')
            .replace(this._prefix, '')
            .replace(this._minusSign, '-')
            .replace(this._decimal, '.')
            .replace(this._numeral, this._index);
        if (filteredText) {
            let parsedValue = +filteredText;
            return isNaN(parsedValue) ? undefined : parsedValue;
        }
        return undefined;
    }
    writeValue(value) {
        this.value = value;
    }
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    getOptions() {
        return {
            localeMatcher: this.localeMatcher,
            style: this.mode,
            currency: this.currency,
            currencyDisplay: this.currencyDisplay,
            useGrouping: this.useGrouping,
            minimumFractionDigits: this.minFractionDigits,
            maximumFractionDigits: this.maxFractionDigits
        };
    }
    getDecimalExpression() {
        const formatter = new Intl.NumberFormat(this.locale, { useGrouping: false });
        return new RegExp(`[${formatter.format(1.1).trim().replace(this._numeral, '')}]`, 'g');
    }
    getGroupingExpression() {
        const formatter = new Intl.NumberFormat(this.locale, { useGrouping: true });
        return new RegExp(`[${formatter.format(1000).trim().replace(this._numeral, '')}]`, 'g');
    }
    getMinusSignExpression() {
        const formatter = new Intl.NumberFormat(this.locale, { useGrouping: false });
        return new RegExp(`[${formatter.format(-1).trim().replace(this._numeral, '')}]`, 'g');
    }
    getCurrencyExpression() {
        if (this.currency) {
            const formatter = new Intl.NumberFormat(this.locale, { style: 'currency', currency: this.currency, currencyDisplay: this.currencyDisplay });
            return new RegExp(`[${formatter.format(1).replace(/\s/g, '').replace(this._numeral, '').replace(this._decimal, '').replace(this._group, '')}]`, 'g');
        }
        return new RegExp(`[]`, 'g');
    }
    filled() {
        return (this.value != null && this.value.toString().length > 0);
    }
    resetRegex() {
        this._numeral.lastIndex = 0;
        this._decimal.lastIndex = 0;
        this._group.lastIndex = 0;
        this._minusSign.lastIndex = 0;
    }
}
/** @nocollapse */ /** @nocollapse */ InputNumber.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: InputNumber, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
/** @nocollapse */ /** @nocollapse */ InputNumber.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.1.2", type: InputNumber, selector: "[inputNumber]", inputs: { showButtons: "showButtons", format: "format", buttonLayout: "buttonLayout", prefix: "prefix", locale: "locale", suffix: "suffix", localeMatcher: "localeMatcher", currency: "currency", currencyDisplay: "currencyDisplay", useGrouping: "useGrouping", disabled: "disabled", inputId: "inputId", styleClass: "styleClass", placeholder: "placeholder", size: "size", maxlength: "maxlength", tabindex: "tabindex", title: "title", ariaLabel: "ariaLabel", ariaRequired: "ariaRequired", name: "name", required: "required", autocomplete: "autocomplete", min: "min", max: "max", minFractionDigits: "minFractionDigits", maxFractionDigits: "maxFractionDigits", precision: "precision", incrementButtonClass: "incrementButtonClass", decrementButtonClass: "decrementButtonClass", incrementButtonIcon: "incrementButtonIcon", decrementButtonIcon: "decrementButtonIcon", mode: "mode", step: "step" }, outputs: { onFocus: "onFocus", onBlur: "onBlur" }, host: { listeners: { "blur": "onModelTouched()" } }, providers: [INPUTNUMBER_VALUE_ACCESSOR], viewQueries: [{ propertyName: "input", first: true, predicate: ["input"], descendants: true }], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: InputNumber, decorators: [{
            type: Directive,
            args: [{
                    selector: '[inputNumber]',
                    providers: [INPUTNUMBER_VALUE_ACCESSOR],
                    host: {
                        '(blur)': 'onModelTouched()'
                    }
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }]; }, propDecorators: { showButtons: [{
                type: Input
            }], format: [{
                type: Input
            }], buttonLayout: [{
                type: Input
            }], prefix: [{
                type: Input
            }], locale: [{
                type: Input
            }], suffix: [{
                type: Input
            }], localeMatcher: [{
                type: Input
            }], currency: [{
                type: Input
            }], currencyDisplay: [{
                type: Input
            }], useGrouping: [{
                type: Input
            }], disabled: [{
                type: Input
            }], inputId: [{
                type: Input
            }], styleClass: [{
                type: Input
            }], placeholder: [{
                type: Input
            }], size: [{
                type: Input
            }], maxlength: [{
                type: Input
            }], tabindex: [{
                type: Input
            }], title: [{
                type: Input
            }], ariaLabel: [{
                type: Input
            }], ariaRequired: [{
                type: Input
            }], name: [{
                type: Input
            }], required: [{
                type: Input
            }], autocomplete: [{
                type: Input
            }], min: [{
                type: Input
            }], max: [{
                type: Input
            }], minFractionDigits: [{
                type: Input
            }], maxFractionDigits: [{
                type: Input
            }], precision: [{
                type: Input
            }], incrementButtonClass: [{
                type: Input
            }], decrementButtonClass: [{
                type: Input
            }], incrementButtonIcon: [{
                type: Input
            }], decrementButtonIcon: [{
                type: Input
            }], mode: [{
                type: Input
            }], step: [{
                type: Input
            }], input: [{
                type: ViewChild,
                args: ['input', { static: false }]
            }], onFocus: [{
                type: Output
            }], onBlur: [{
                type: Output
            }] } });
class InputNumberModule {
}
/** @nocollapse */ /** @nocollapse */ InputNumberModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: InputNumberModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ /** @nocollapse */ InputNumberModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: InputNumberModule, declarations: [InputNumber], imports: [CommonModule], exports: [InputNumber] });
/** @nocollapse */ /** @nocollapse */ InputNumberModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: InputNumberModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: InputNumberModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    exports: [InputNumber],
                    declarations: [InputNumber]
                }]
        }] });

const Data = { loading: false, counts: 0 };
class ServiwLoadingComponent {
    /**
     * Gets loading
     */
    get loading() {
        return Data.loading;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwLoadingComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLoadingComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ /** @nocollapse */ ServiwLoadingComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.1.2", type: ServiwLoadingComponent, selector: "serviw-loading", ngImport: i0, template: "<div class=\"loading\" *ngIf=\"loading\">\n    <div class=\"l-center\">\n        <div class=\"lds-roller\">\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n        </div>\n    </div>\n</div>", styles: [".loading{position:absolute;background-color:#3339;width:100%;height:100%;z-index:100000}.l-center{position:absolute;top:50%;left:50%;transform:translate(-50%,-50%)}.lds-roller{display:inline-block;position:relative;width:64px;height:64px}.lds-roller div{animation:lds-roller 1.2s cubic-bezier(.5,0,.5,1) infinite;transform-origin:32px 32px}.lds-roller div:after{content:\" \";display:block;position:absolute;width:6px;height:6px;border-radius:50%;background:#fff;margin:-3px 0 0 -3px}.lds-roller div:nth-child(1){animation-delay:-36ms}.lds-roller div:nth-child(1):after{top:50px;left:50px}.lds-roller div:nth-child(2){animation-delay:-72ms}.lds-roller div:nth-child(2):after{top:54px;left:45px}.lds-roller div:nth-child(3){animation-delay:-.108s}.lds-roller div:nth-child(3):after{top:57px;left:39px}.lds-roller div:nth-child(4){animation-delay:-.144s}.lds-roller div:nth-child(4):after{top:58px;left:32px}.lds-roller div:nth-child(5){animation-delay:-.18s}.lds-roller div:nth-child(5):after{top:57px;left:25px}.lds-roller div:nth-child(6){animation-delay:-.216s}.lds-roller div:nth-child(6):after{top:54px;left:19px}.lds-roller div:nth-child(7){animation-delay:-.252s}.lds-roller div:nth-child(7):after{top:50px;left:14px}.lds-roller div:nth-child(8){animation-delay:-.288s}.lds-roller div:nth-child(8):after{top:45px;left:10px}@keyframes lds-roller{0%{transform:rotate(0)}to{transform:rotate(360deg)}}\n"], directives: [{ type: i1$1.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLoadingComponent, decorators: [{
            type: Component,
            args: [{ selector: 'serviw-loading', template: "<div class=\"loading\" *ngIf=\"loading\">\n    <div class=\"l-center\">\n        <div class=\"lds-roller\">\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n            <div></div>\n        </div>\n    </div>\n</div>", styles: [".loading{position:absolute;background-color:#3339;width:100%;height:100%;z-index:100000}.l-center{position:absolute;top:50%;left:50%;transform:translate(-50%,-50%)}.lds-roller{display:inline-block;position:relative;width:64px;height:64px}.lds-roller div{animation:lds-roller 1.2s cubic-bezier(.5,0,.5,1) infinite;transform-origin:32px 32px}.lds-roller div:after{content:\" \";display:block;position:absolute;width:6px;height:6px;border-radius:50%;background:#fff;margin:-3px 0 0 -3px}.lds-roller div:nth-child(1){animation-delay:-36ms}.lds-roller div:nth-child(1):after{top:50px;left:50px}.lds-roller div:nth-child(2){animation-delay:-72ms}.lds-roller div:nth-child(2):after{top:54px;left:45px}.lds-roller div:nth-child(3){animation-delay:-.108s}.lds-roller div:nth-child(3):after{top:57px;left:39px}.lds-roller div:nth-child(4){animation-delay:-.144s}.lds-roller div:nth-child(4):after{top:58px;left:32px}.lds-roller div:nth-child(5){animation-delay:-.18s}.lds-roller div:nth-child(5):after{top:57px;left:25px}.lds-roller div:nth-child(6){animation-delay:-.216s}.lds-roller div:nth-child(6):after{top:54px;left:19px}.lds-roller div:nth-child(7){animation-delay:-.252s}.lds-roller div:nth-child(7):after{top:50px;left:14px}.lds-roller div:nth-child(8){animation-delay:-.288s}.lds-roller div:nth-child(8):after{top:45px;left:10px}@keyframes lds-roller{0%{transform:rotate(0)}to{transform:rotate(360deg)}}\n"] }]
        }] });
const SERVIW_LOADING = new InjectionToken('app.config', {
    providedIn: 'root', factory: () => ServiwLoading
});
// Loading
const ServiwLoading = {
    active: Data.loading,
    set: (val) => {
        Data.loading = val;
    },
    multiple: (val) => {
        Data.counts = val ? Data.counts + 1 : Data.counts - 1;
        if (Data.counts <= 0) {
            Data.loading = false;
            Data.counts = 0;
        }
        return Data.loading;
    }
};

class ServiwLoadingModule {
}
/** @nocollapse */ /** @nocollapse */ ServiwLoadingModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLoadingModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ /** @nocollapse */ ServiwLoadingModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLoadingModule, declarations: [ServiwLoadingComponent], imports: [CommonModule], exports: [ServiwLoadingComponent] });
/** @nocollapse */ /** @nocollapse */ ServiwLoadingModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLoadingModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLoadingModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [ServiwLoadingComponent],
                    entryComponents: [ServiwLoadingComponent],
                    exports: [ServiwLoadingComponent],
                    imports: [CommonModule]
                }]
        }] });

class ServiwGrowl {
    /**
     * Creates an instance of serviw growl.
     */
    constructor() {
        this.messages = this.allMessages = [];
        this.sidebar = false;
        this.messagesSub = new Subject();
        this.icons = { success: 'check', info: 'info-circle', error: 'close', warn: 'exclamation-triangle' };
    }
    /**
     * Sets icons
     * @param icons
     */
    setIcons(icons) {
        for (let name in icons)
            if (icons[name])
                this.icons[name] = icons[name];
    }
    /**
     * Subscribes serviw growl
     * @returns
     */
    get events() {
        return this.messagesSub.asObservable();
    }
    /**
      ** Visualizar u ocultal mensajes de la barra lateral
      */
    displayMessages() {
        this.sidebar = !this.sidebar;
    }
    /**
     * Eliminar un mensaje al asignarse el index, o resetear la lista
     *
     * @param i Index mensaje en array
     */
    clear(i = -1) {
        if (i && i >= 0 || i < this.allMessages.length)
            this.allMessages.splice(i, 1);
        else
            this.allMessages.pop();
    }
    /**
     * * Agregar nuevo mensaje
     * @param message
     */
    addMessage(message) {
        this.allMessages.unshift(message);
        this.messagesSub.next(message);
    }
    /**
     * * Visualizar un mensaje
     * @param message
     */
    show(message) {
        this.messages.pop();
        this.messages.push(message);
        this.allMessages.unshift(message);
        this.messagesSub.next(message);
    }
    /**
     * Success serviw growl
     * @param summary
     * @param detail
     */
    success(summary, detail, duration) {
        this.show({ summary, detail, severity: 'success', icon: this.icons.success, duration });
    }
    /**
     * Adds success
     * @param summary
     * @param detail
     */
    addSuccess(summary, detail, duration) {
        const message = { detail, summary, severity: 'success', icon: this.icons.success, duration };
        this.addMessage(message);
    }
    /**
     * Infos serviw growl
     * @param summary
     * @param detail
     */
    info(summary, detail = '', duration) {
        this.show({ summary, detail, severity: 'info', icon: this.icons.info, duration });
    }
    /**
     * Adds info
     * @param summary
     * @param detail
     */
    addInfo(summary, detail, duration) {
        const message = { summary, detail, severity: 'info', icon: this.icons.info, duration };
        this.addMessage(message);
    }
    /**
     * Errors message request
     * @param error
     * @param [summary]
     * @returns message request
     */
    infoMessageRequest(error, summary = 'Información') {
        const errorException = error.json();
        const data = errorException.data;
        if (ServiwRest.BAD_REQUEST() === error.status) {
            const e = errorException;
            this.info(summary, e.message);
        }
        else {
            this.errorMessage(error);
        }
        return data;
    }
    /**
     * Errors serviw growl
     * @param summary
     * @param detail
     */
    error(summary, detail = '', duration) {
        this.show({ summary, detail, severity: 'error', icon: this.icons.error, duration });
    }
    /**
     * Adds error
     * @param summary
     * @param detail
     */
    addError(summary, detail, duration) {
        const message = { summary, detail, severity: 'error', icon: this.icons.error, duration };
        this.addMessage(message);
    }
    /**
     * Warns serviw growl
     * @param summary
     * @param detail
     */
    warn(summary, detail, duration) {
        this.show({ summary, detail, severity: 'warn', icon: this.icons.warn, duration });
    }
    /**
     * Adds warn
     * @param summary
     * @param detail
     */
    addWarn(summary, detail, duration) {
        const message = { summary, detail, severity: 'warn', icon: this.icons.warn, duration };
        this.addMessage(message);
    }
    /**
     * Errors message request
     * @param error
     * @param [summary]
     * @returns message request
     */
    errorMessageRequest(error, summary = 'Error') {
        const errorException = error.json();
        const data = errorException.data;
        if (ServiwRest.BAD_REQUEST() == error.status) {
            const e = errorException;
            this.error(summary, e.message);
        }
        else {
            this.errorMessage(error);
        }
        return data;
    }
    /**
     * Errors message
     * @param error
     * @returns message
     */
    errorMessage(error) {
        let data;
        const msjError = { summary: 'Error', detail: '', severity: 'error', icon: this.icons.error, duration: 2000, data: error };
        try {
            if (error instanceof HttpErrorResponse)
                data = error;
            else
                data = JSON.parse(error);
        }
        catch (e) {
            msjError.detail = 'Error del sistema, inesperado';
            this.show(msjError);
            return e;
        }
        if (data.ok)
            msjError.detail = 'No se recibió información desde el servidor';
        else if (data.status == 401)
            msjError.detail = 'Error de autenticación';
        else if (data.status >= 400 && data.status < 500)
            msjError.detail = data.message;
        else if (data.status >= 500)
            msjError.detail = 'Se presentó un error en el servidor,  por favor contacta a soporte técnico';
        else
            msjError.detail = 'No hay conexión';
        this.show(msjError);
        return null;
    }
    /**
     * Saved correctly
     */
    savedCorrectly() {
        this.success('Guardado', 'correctamente');
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwGrowl.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwGrowl, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwGrowl.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwGrowl });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwGrowl, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return []; } });

/**
 * Svw json events
 * @template I Entity interface
 */
class SvwJsonEvents {
    /**
     * Creates an instance of svw json events.
     * @param schema
     */
    constructor(schema) {
        this.schema = schema;
        this.loading = false;
    }
    /** Set entity Json */
    set json(val) {
        if (ServiwUtil.empty(val))
            return;
        switch (this.schema) {
            case SCHEMA_GRAPHQL:
                this._entity = new EntityGraphql(val);
                this.nameEntity = this._entity.json.singular;
                break;
            case SCHEMA_REST:
                this._entity = new EntityRest(val);
                this.nameEntity = this._entity.json.nameEntity;
                break;
            default:
                throw new Error('Esquema no válido');
        }
        this.nameEntity = this.nameEntity ? this.nameEntity.toLowerCase() : 'serviw-entity';
        this.onLoadJson();
    }
    /** Get Json */
    get json() {
        return this.entity.json;
    }
    /** Get json original */
    get jsonI() {
        return this.json;
    }
    /** Get entity */
    get entity() {
        return this._entity;
    }
}
/** @nocollapse */ /** @nocollapse */ SvwJsonEvents.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: SvwJsonEvents, deps: [{ token: {} }], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ /** @nocollapse */ SvwJsonEvents.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.1.2", type: SvwJsonEvents, selector: "ng-component", inputs: { nameEntity: "nameEntity", json: "json" }, ngImport: i0, template: '', isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: SvwJsonEvents, decorators: [{
            type: Component,
            args: [{
                    template: ''
                }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [{}]
                }] }]; }, propDecorators: { nameEntity: [{
                type: Input
            }], json: [{
                type: Input
            }] } });

/**
 * Serviw core form
 */
class ServiwCoreForm extends SvwJsonEvents {
    constructor(serviwGrowl, serviwQueryService, serviwForm, cd) {
        super(serviwQueryService.settings.scheme);
        this.serviwGrowl = serviwGrowl;
        this.serviwQueryService = serviwQueryService;
        this.serviwForm = serviwForm;
        this.cd = cd;
        /** Event on beforeSave */
        this.beforeSave = new EventEmitter();
        /** Event on submitForm */
        this.afterSave = new EventEmitter();
        /** onLoadForm form ServiwForm<I> */
        this.onLoadForm = new EventEmitter();
        this.create = true;
        this.queryKey = false;
        this.showMessageAfterSaving = true;
    }
    /**
     * Input  of serviw form component
     */
    set value(data) {
        this._value = data;
        if (!ServiwUtil.empty(data) && this.formGroup)
            this.formGroup.setValue(ServiwFormService.refactorSetDataForm(data, this.entity.fields, this.formGroup));
        this.updateCreate();
    }
    /**
     * Gets value
     */
    get value() {
        if (this.formGroup)
            return this.formGroup.value;
        return ServiwUtil.emptyObject();
    }
    /**
     * Gets call on after save
     */
    get callOnAfterSave() {
        const valueForm = this.entity.schema == SCHEMA_GRAPHQL ? ServiwFormService.refactorDataForm(this.formGroup?.value, this.entity) : this.formGroup?.value;
        const callData = {
            value: valueForm,
            continue: (value) => { return this.save(value); }
        };
        return callData;
    }
    /**
     * Determines whether load json on
     */
    onLoadJson() {
        this.formGroup = this.serviwForm.getForm(this.entity, true);
        this.cd.detectChanges();
        this.updateCreate();
        this.value = this._value;
        this.onLoadForm.emit(this.formGroup);
    }
    /**
     * after content init
     */
    ngAfterContentInit() {
        if (this.queryKey && this.entity)
            this.querySingle();
    }
    /**
     * Updates create
     */
    updateCreate() {
        this.create = this.entity.emptyId(this.value);
    }
    /**
     * Querys single
     */
    querySingle() {
        // 1.
        if (this.create)
            return;
        this.entity.setValueId(this.value);
        const query = this.serviwQueryService.querySingle(this.entity);
        if (!query)
            return;
        // 3.
        const getOk = (response) => {
            // 3.1.1.
            this.value = query.refactor(response);
            this.entity.resetValueId();
            this.loading = false;
        };
        const getError = (error) => {
            // 3.1.1.
            this.loading = false;
            this.serviwGrowl.errorMessage(error);
        };
        // 4.
        query.http().then(getOk, getError);
    }
    /**
     * Views form
     */
    viewForm() {
    }
    /**
     * Determines whether submit on
     * @param event
     */
    onSubmit(event) {
        // this.saveEvent();
    }
    /**
     * Resibe eventos on click de las acciones del formulaio
     *
     * 1. Evento para guardar, Define función a emitir
     * @param event
     */
    saveEvent() {
        // 1.
        if (this.formGroup?.valid)
            if (this.beforeSave.observers.length > 0)
                this.beforeSave.emit(this.callOnAfterSave);
            else
                this.save();
    }
    /**
     * Guardar datos de un formulario
     *
     * 1. loading true, define esquema de los datos
     * 2. Callbacks, del servidor
     *  2.1.1. Actualizar formulario, loading false
     *  2.2.1.
     */
    save(value) {
        //  1.
        this.loading = true;
        const query = this.serviwQueryService.querySave(this.entity, this.formGroup);
        if (value) {
            if (query.data)
                query.data.variables = value;
            else
                query.data = {
                    variables: value
                };
        }
        // 2.
        const ok = (data) => {
            // 2.1.1.
            this.loading = false;
            const dataSave = query.updateForm ? query.updateForm(data) : {};
            const error = dataSave || !query.getError ? '' : query.getError(data);
            const viewError = (detail = error) => error && this.serviwGrowl.error(this.json.titleForm || 'Error', detail);
            const afterSave = { value: dataSave, create: this.create, errors: query.getErrors ? query.getErrors(data) : '', error, viewError };
            if (!afterSave.error) {
                this.value = dataSave;
                if (this.showMessageAfterSaving)
                    this.serviwGrowl.success(this.json.titleForm || 'Registro', 'Guardado correctamente');
            }
            if (this.afterSave.observers.length > 0)
                this.afterSave.emit(afterSave);
            else if (afterSave.error)
                viewError();
        };
        const error = (error) => {
            // 2.2.1.
            this.loading = false;
            this.serviwGrowl.errorMessage(error);
        };
        if (!query.http)
            throw 'No se definió un cliente http para guardar';
        const response = query.http();
        response.then(ok, error);
        return response;
    }
    /**
     * on destroy
     */
    ngOnDestroy() {
        if (this.formGroup?.subscription)
            this.formGroup.subscription.unsubscribe();
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwCoreForm.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreForm, deps: [{ token: ServiwGrowl }, { token: ServiwQueryService }, { token: ServiwFormService }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ /** @nocollapse */ ServiwCoreForm.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.1.2", type: ServiwCoreForm, selector: "ng-component", inputs: { queryKey: "queryKey", showMessageAfterSaving: "showMessageAfterSaving", value: "value" }, outputs: { beforeSave: "beforeSave", afterSave: "afterSave", onLoadForm: "onLoadForm" }, usesInheritance: true, ngImport: i0, template: '', isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreForm, decorators: [{
            type: Component,
            args: [{
                    template: ''
                }]
        }], ctorParameters: function () { return [{ type: ServiwGrowl }, { type: ServiwQueryService }, { type: ServiwFormService }, { type: i0.ChangeDetectorRef }]; }, propDecorators: { queryKey: [{
                type: Input
            }], showMessageAfterSaving: [{
                type: Input
            }], beforeSave: [{
                type: Output
            }], afterSave: [{
                type: Output
            }], onLoadForm: [{
                type: Output
            }], value: [{
                type: Input
            }] } });

class ServiwEvents {
    /**
     * Creates an instance of serviw events.
     */
    constructor() {
        this.events = {};
        this.subscribes = {};
    }
    /**
     * Events emitter
     * @template I
     * @param name
     * @returns emitter
     */
    eventEmitter(name) {
        return this.events[name];
    }
    /**
     * Subscribes serviw events
     * @param name
     * @param call
     * @param [only]
     * @returns subscribe
     */
    subscribe(name, call, only = false) {
        if (only && this.subscribes[name])
            return this.subscribes[name];
        this.subscribes[name] = this.event(name).subscribe(call);
        return this.subscribes[name];
    }
    /**
     * Events serviw events
     * @param name
     * @returns event
     */
    event(name) {
        const empty = ServiwUtil.empty(this.events[name]);
        if (empty)
            this.events[name] = new EventEmitter();
        return this.events[name].asObservable();
    }
    /**
     * Dispatchs serviw events
     * @param name
     * @param value
     * @returns true if dispatch
     */
    dispatch(name, value) {
        const valid = !ServiwUtil.empty(this.events[name]);
        if (valid)
            this.events[name].emit(value);
        return valid;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwEvents.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwEvents, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwEvents.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwEvents, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwEvents, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });

class ServiwCoreTable extends SvwJsonEvents {
    /**
     * Creates an instance of serviw core table.
     * @param serviwQuery
     * @param serviwEvents
     */
    constructor(serviwQuery, serviwEvents, serviwGrowl) {
        super(serviwQuery.settings.scheme);
        this.serviwQuery = serviwQuery;
        this.serviwEvents = serviwEvents;
        this.serviwGrowl = serviwGrowl;
        this.lazy = false;
        this.loading = false;
        this.onSelect = new EventEmitter();
        this._onCheck = new EventEmitter();
        this.onChangeData = new EventEmitter();
        this.onFilter = new EventEmitter();
        this.onDelete = new EventEmitter();
        this.data = [];
        this.rowSelect = ServiwUtil.emptyObject();
    }
    /**
     * Determines whether load json on
     */
    onLoadJson() {
        if (!this.entity)
            throw new Error('Error al cargar JSON');
        this.loadFormFilters();
    }
    /**
    * after content init
    */
    ngAfterContentInit() {
        if (!this.lazy)
            this.update();
    }
    /**
     * Loads form filters
     */
    loadFormFilters() {
        this.formFilters = undefined;
        const controls = ServiwUtil.emptyObject();
        this.entity.mapFields((f) => {
            if (f.property.filterType) {
                controls[f.nameField] = new FormControl();
                if (f.property.template == 'date')
                    controls[`${f.nameField}_before`] = new FormControl();
            }
        });
        if (Object.keys(controls).length > 0)
            this.formFilters = new ServiwForm(controls);
    }
    /**
     * Filters serviw core table
     */
    filterTable() {
        if (!this.formFilters?.controls)
            return;
        const keys = Object.keys(this.formFilters?.controls);
        for (const key of keys)
            if (this.json.fields[key])
                this.filterField(key, this.json.fields[key]);
        this.updateFilters();
    }
    /**
     * Resets filters
     */
    resetFilters() {
        if (this.formFilters)
            this.formFilters.reset();
        this.entity.resetParams();
        this.updateFilters();
    }
    /**
     * Updates filters
     */
    updateFilters() {
        const eventFilter = { params: this.entity.params };
        if (this.onFilter.observers.length > 0) {
            eventFilter.continue = (params) => {
                if (params)
                    this.entity.params = params;
                this.update();
            };
            this.onFilter.emit(eventFilter);
        }
        else
            this.update();
    }
    /**
     * Filters field
     * @param property
     */
    filterField(key, property) {
        let value = this.formFilters?.value[key];
        let type = property.type;
        const forceType = property.template == 'date';
        if (property.type == 'int')
            value = Number.parseInt(value);
        else if (property.type == 'float' || property.type == 'double')
            value = Number.parseFloat(value);
        else if (property.type == 'date' || property.template == 'date') {
            const before = this.formFilters?.value[`${key}_before`];
            if (value || before)
                value = { after: value, before };
            if (this.entity.schema == SCHEMA_GRAPHQL)
                type = `${this.entity.json.singular}Filter_${key}`;
        }
        switch (this.entity.schema) {
            case SCHEMA_GRAPHQL:
                if (value)
                    this.entity.params[key] = { type, value, forceType };
                else
                    delete this.entity.params[key];
                break;
            case SCHEMA_REST:
                if (this.entity.params.filters)
                    this.entity.params.filters[key] = value;
                else
                    this.entity.params.filters = {
                        [key]: value
                    };
                break;
        }
    }
    /**
     * Filters table
     * @param filter
     * @returns table
     * @deprecated
     */
    setFilterTable(filter) {
        const field = this.jsonI.fields ? this.jsonI.fields[filter.column] : undefined;
        if (!field)
            throw `No se ha definido el filtro para ${filter.column}`;
        let eventFilter = {};
        let value = filter.value;
        if (field.type == 'int')
            value = Number.parseInt(value);
        else if (field.type == 'float' || field.type == 'double')
            value = Number.parseFloat(value);
        switch (this.entity.schema) {
            case SCHEMA_GRAPHQL:
                if (value)
                    this.entity.params[filter.column] = { type: field.type, value: value };
                else
                    delete this.entity.params[filter.column];
                eventFilter.params = this.entity.params;
                break;
            case SCHEMA_REST:
                this.entity.params.filters[filter.column] = value;
                eventFilter.params = this.entity.params;
                break;
            default:
                break;
        }
        if (this.timeFilter)
            clearTimeout(this.timeFilter);
        const callFilter = () => {
            if (this.onFilter.observers.length > 0) {
                eventFilter.continue = (params) => {
                    if (params)
                        this.entity.params = params;
                    this.update();
                };
                this.onFilter.emit(eventFilter);
            }
            else
                this.update();
        };
        this.timeFilter = setTimeout(callFilter, this.json.filterDelay || 1000);
    }
    /**
     * Sets data
     * @param vals
     */
    setData(vals) {
        this.data = vals || [];
        this.onRowSelect();
        this.onChangeData.emit(this.data);
    }
    /**
     * Determines whether row select on
     * @param row
     */
    onRowSelect(row = ServiwUtil.emptyObject()) {
        const empty = {};
        this.rowSelect = row || empty;
        this.onSelect.emit(row);
    }
    /**
     * Updates serviw core table component
     */
    update(item) {
        if (item) {
            const setItem = ServiwUtil.isObject(item) ? item : ServiwUtil.emptyObject();
            this.onRowSelect(setItem);
            return this.setData(ServiwUtil.updateArray(this.data, item, this.entity.idName));
        }
        else {
            for (const name in this.json.filters)
                if (this.entity.schema == SCHEMA_GRAPHQL) {
                    let filter = this.json.filters[name];
                    this.entity.params[name] = ServiwUtil.isObject(this.json.filters[name]) ? this.json.filters[name] : ServiwGraphqlService.getFieldFilter(this.entity.fields[name], this.json.filters[name]);
                }
                else
                    this.entity.params.filters[name] = this.entity.json.filters[name];
            this.getData();
        }
        this.onRowSelect(ServiwUtil.emptyObject());
    }
    /**
     * Updates data
     * @returns
     */
    updateData() {
        return this.getData();
    }
    /**
     * Gets data
     * @returns Promise | null
     */
    getData() {
        const query = this.serviwQuery.getData(this.entity);
        this.loading = true;
        this.disableFilters(true);
        const dataOk = (response) => {
            this.loading = false;
            this.disableFilters(false);
            try {
                const data = query.refactor(response);
                if (data) {
                    this.setData(data);
                    this.onGetData(response, query);
                    if (this.json.emptyDataMessage != false && this.data.length == 0)
                        this.serviwGrowl.info(this.entity.json.titleTable, 'Sin resultados');
                }
                else {
                    let error = query?.getError ? query.getError(response) : '';
                    this.serviwGrowl.error(this.json.titleTable, error);
                }
            }
            catch (error) {
                const errorCatch = { message: error };
                dataError(errorCatch);
            }
        };
        const dataError = (error) => {
            this.loading = false;
            this.disableFilters(false);
            this.serviwGrowl.errorMessage(error);
            this.setData([]);
        };
        const promise = query.http();
        promise.then(dataOk, dataError);
        return promise;
    }
    /**
     * Disables filters
     * @param value
     */
    disableFilters(value) {
        if (this.formFilters)
            if (value)
                this.formFilters.disable();
            else
                this.formFilters.enable();
    }
    /**
     * Removes ite
     */
    deleteItem() {
        const query = this.serviwQuery.deleteItem(this.entity, this.rowSelect);
        const idValue = this.rowSelect[this.entity.idName];
        this.loading = true;
        const callOk = (response) => {
            this.loading = false;
            const data = query.refactor(response);
            if (!data)
                return this.serviwGrowl.error('Error al eliminar ' + (this.json.titleForm || ''), 'Se ha presentado un problema al eliminar el registro');
            this.serviwGrowl.info(this.json.titleTable || 'Registro', 'Se ha eliminado el registro');
            const index = this.data.findIndex((i) => i[this.entity.idName] === idValue);
            if (index !== -1) {
                const data = [...this.data];
                data.splice(index, 1);
                this.setData(data);
            }
            else
                this.update();
            this.onDelete.emit(data);
        };
        const dataError = (error) => {
            this.loading = false;
            this.onDelete.emit(null);
            this.serviwGrowl.errorMessage(error);
        };
        const promise = query.http();
        promise.then(callOk, dataError);
        return promise;
    }
    /**
     * Emits check
     * @param [empty]
     * @returns true if check
     */
    emitCheck(empty = false) {
        if (!empty && ServiwUtil.empty(this.rowSelect))
            return false;
        this._onCheck.emit(this.rowSelect);
        return this.inCheck;
    }
    /**
     * Rows equals
     * @param row
     * @returns equals
     */
    rowEquals(row) {
        const select = ServiwUtil.objEquals(row, this.rowSelect, { [this.entity.idName]: '' }) ? 'selected-row' : '';
        return select;
    }
    /**
     * Gets whether on check
     */
    get onCheck() {
        return this._onCheck.asObservable();
    }
    /**
     * Gets whether in check
     */
    get inCheck() {
        return this._onCheck.observers.length > 0;
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwCoreTable.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreTable, deps: [{ token: ServiwQueryService }, { token: ServiwEvents }, { token: ServiwGrowl }], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ /** @nocollapse */ ServiwCoreTable.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.1.2", type: ServiwCoreTable, selector: "ng-component", inputs: { lazy: "lazy" }, outputs: { onSelect: "onSelect", onChangeData: "onChangeData", onFilter: "onFilter", onDelete: "onDelete" }, usesInheritance: true, ngImport: i0, template: '', isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwCoreTable, decorators: [{
            type: Component,
            args: [{
                    template: ''
                }]
        }], ctorParameters: function () { return [{ type: ServiwQueryService }, { type: ServiwEvents }, { type: ServiwGrowl }]; }, propDecorators: { onSelect: [{
                type: Output
            }], onChangeData: [{
                type: Output
            }], onFilter: [{
                type: Output
            }], onDelete: [{
                type: Output
            }], lazy: [{
                type: Input
            }] } });

class ServiwParams {
    /**
     * Creates an instance of serviw params service.
     */
    constructor() {
        this.data = this.permission = {};
        this.development = false;
        this.userEvents = new Subject();
        this.resetAll();
    }
    /**
     * Retorna un elemento de data clonado
     *
     * 1. Obtener objeto de data
     * 2. Realiza proceso de clonación
     *
     * @param clase nombre de la clase u objeto
     */
    get(clase) {
        // 1.
        const val = this.data[clase] || {};
        // 2.
        const clone = ServiwUtil.cloneObject(val);
        return clone;
    }
    /**
     * Validar si existe un parametro en data
     *
     * @param param
     */
    validParam(param) {
        let valid = this.data[param] != undefined && this.data[param] != null;
        return valid;
    }
    /**
     * Validar si existe un permiso
     * @param permission
     */
    validPermission(permission) {
        let valid = this.permission[permission] == true;
        return valid;
    }
    /**
     * Set user details
     */
    setUser(val) {
        if (typeof val == 'object' && val) {
            const user = val || {};
            this.userDetails = user;
            this.authenticated = user.fullName || '';
            this.userEvents.next(user);
        }
        else
            console.warn('Valor no asignado');
    }
    /**
     * Returns serviw params
     * @param [emitEmpty] emit userEvents
     * @returns  T
     */
    getUser(emitEmpty = true) {
        if (emitEmpty && ServiwUtil.empty(this.userDetails))
            this.userEvents.next({});
        return this.userDetails;
    }
    /**
     * Resets all
     */
    resetAll() {
        ServiwUtil.resetObject(this.data);
        ServiwUtil.resetObject(this.permission);
        this.setUser({});
        this.authenticated = '';
        this.loading = false;
    }
    /**
     * Gets user events
     */
    get changeUser() {
        return this.userEvents.asObservable();
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwParams.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwParams, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwParams.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwParams, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwParams, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return []; } });

const JSON_STORAGE = new InjectionToken('app.config', { providedIn: 'root', factory: () => DataApp.storage });
class ServiwJsonStorage {
    constructor(serviwRest, serviwGrowl) {
        this.serviwRest = serviwRest;
        this.serviwGrowl = serviwGrowl;
        this.loading = false;
        this._onLoad = new EventEmitter();
    }
    /**
     * Gets storage
     */
    get data() {
        return DataApp.storage || ServiwUtil.emptyObject();
    }
    /**
     * Gets whether is valid
     */
    get isValid() {
        return DataApp.valid();
    }
    /**
     * Sets storage
     * @template S Storage
     * @param data
     */
    setStorage(data) {
        DataApp.storage = data;
    }
    /**
     * Loads json storage service
     * @param [path] = serviw/schema
     * @param [concatApi] = true
     */
    load(path = 'serviw/schema', concatApi = true) {
        if (this.promise)
            return this.promise;
        const url = concatApi ? this.serviwRest.settings.API + path : path;
        return this.loadStorage(url);
    }
    /**
     * Load storage
     * @param url
     */
    loadStorage(url) {
        this.loading = true;
        const getOk = (json) => {
            DataApp.storage = json;
            this._onLoad.next(DataApp.storage);
            this.loading = false;
            this.promise = undefined;
        };
        const getError = (error) => {
            this.loading = false;
            this.serviwGrowl.errorMessage(error);
            console.error('Error al cargar esquema json');
            this.promise = undefined;
        };
        this.promise = this.serviwRest.get(url);
        this.promise.then(getOk, getError);
        return this.promise;
    }
    /**
     * Gets whether on load
     */
    get onLoad() {
        return this._onLoad.asObservable();
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwJsonStorage.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwJsonStorage, deps: [{ token: ServiwRest }, { token: ServiwGrowl }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ServiwJsonStorage.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwJsonStorage, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwJsonStorage, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: ServiwRest }, { type: ServiwGrowl }]; } });

class AuthServiceInterface {
}

// por default
const isExpiredToken = () => true;
const getNow = () => {
    return Number.parseInt((Date.parse(new Date()) + '').slice(0, 10));
};
const getExpiredToken = (expired = 0) => {
    return () => getNow() >= expired;
};
const SERVIW_USER = new InjectionToken('app.config', {
    providedIn: 'root', factory: () => userObservable
});
const UserSubjec = new BehaviorSubject({ isExpiredToken });
const userObservable = UserSubjec.asObservable();
// ----------- JWT OPTIONS
const AuthJWTOptionsDefault = {
    endpointRefresh: 'api/token/refresh',
    messageError: 'Error al refrescar el token',
    fieldName: 'token',
    fieldRefresh: 'refresh_token',
    percentageToRefresh: 10
};

class AuthJtwTokenService extends AuthServiceInterface {
    /**
     * Creates an instance of user service.
     * @param serviwRest
     * @param serviwGrowl
     */
    constructor(serviwRest, serviwGrowl, router, authOptions) {
        super();
        this.serviwRest = serviwRest;
        this.serviwGrowl = serviwGrowl;
        this.router = router;
        this.authOptions = authOptions;
        /** Timer refresh token of auth service */
        this.timerRefreshToken = null;
        /** Init */
        this.init = false;
        this.keyNameToken = 'TOKEN_' + serviwRest.settings.NAME_APP.toUpperCase().replace(/ /g, '_');
    }
    /** API */
    get api() { return this.serviwRest.settings.API; }
    /** percentageToRefresh  */
    get percentRefresh() { return this.authOptions.token?.percentageToRefresh || 10; }
    /**
     * Redirects login
     */
    redirectLogin() {
        this.router.navigateByUrl(this.authOptions.login.path);
    }
    /**
     * Logins user service
     * @param username
     * @param password
     * @returns
     */
    login(data) {
        const observable = from(this.serviwRest.post(`${this.api}${this.authOptions.login.endpoint}`, data));
        return this.loadToken(observable, true);
    }
    /**
     * Authentications auth jtw token service
     * @template I
     * @returns authentication
     */
    authentication() {
        this.init = true;
        const token = this.getToken();
        const opts = this.authOptions;
        if (!token || !opts.token) {
            throw 'Token para autenticar no definido';
        }
        if (!this.configToken(token[opts.token?.fieldName]))
            return Promise.resolve(false);
        if (opts.authentication.endpoint === opts.token?.endpointRefresh) {
            if (this.tokenToRefresh(token, true))
                return this.refreshToken(token);
        }
        const observable = from(this.serviwRest.get(`${this.api}${opts.authentication.endpoint}`));
        let ob = observable.pipe(map(response => {
            const user = this.getUser();
            user[this.authOptions.authentication.fieldInUser] = response;
            this.setUser(user);
            return true;
        }), catchError(error => {
            this.resetUser();
            this.serviwGrowl.errorMessage(error);
            return of(false);
        }));
        return lastValueFrom(ob);
    }
    /**
     * Refreshs token
     */
    refreshToken(token) {
        if (!this.authOptions.token?.fieldRefresh && !this.authOptions.token?.fieldRefresh) {
            throw 'No se encuentra parametrizado correctamente el token';
        }
        this.resetTimer();
        const data = { [this.authOptions.token.fieldRefresh]: token[this.authOptions.token.fieldRefresh] };
        const observable = from(this.serviwRest.post(`${this.api}${this.authOptions.token.endpointRefresh}`, data));
        return this.loadToken(observable, false);
    }
    /**
     * Loads token
     * @param observable
     * @returns
     */
    loadToken(observable, login) {
        let ob = observable.pipe(map(response => this.refactorJtwUser(response)), catchError(error => {
            const messaje = this.authOptions.login.messageError;
            login && messaje ? this.serviwGrowl.error('Error en la autenticación', messaje) : this.serviwGrowl.errorMessage(error);
            if (!login)
                this.resetUser();
            return of({});
        })).pipe(tap((user) => {
            if (login && user[this.authOptions.userIdentityField] && this.authOptions.authentication.callAfterLogin)
                this.authentication();
        }));
        return lastValueFrom(ob);
    }
    /**
     * Sets user
     * @param response
     * @returns
     */
    refactorJtwUser(response) {
        let user = this.decodeJwt(response.token || '');
        user.isExpiredToken = getExpiredToken(user.exp);
        localStorage.setItem(this.keyNameToken, JSON.stringify(response));
        this.setUser(user);
        if (!this.authOptions.token?.fieldName)
            throw 'No se ha definido un token';
        this.configToken(response[this.authOptions.token?.fieldName]);
        if (this.tokenToRefresh(response)) {
            const call = () => this.refreshToken(response);
            const exp = (user.exp || 0);
            const total = exp - (user.iat || 0);
            let percent = total * this.percentRefresh / 100;
            const falta = exp - getNow();
            if (falta < percent)
                call();
            else {
                percent = falta - (falta * this.percentRefresh / 100);
                this.timerRefreshToken = setInterval(call, percent * 1000);
            }
        }
        return user;
    }
    /**
     * Sets user
     * @param user
     */
    setUser(user) {
        UserSubjec.next(user);
    }
    /**
     * Tokens to refresh
     * @param token
     * @param [force]
     * @returns
     */
    tokenToRefresh(token, force = false) {
        if (!token || !this.authOptions.token?.fieldRefresh)
            throw 'No se ha definido toda la información para refrescar el token';
        if (token[this.authOptions.token?.fieldRefresh] && (force || !this.timerRefreshToken))
            return true;
        return false;
    }
    /**
     * Configs token
     */
    configToken(token) {
        const headers = { 'Content-Type': 'application/json' };
        if (token)
            headers['Authorization'] = 'Bearer ' + token;
        const data = { responseType: 'json', headers: new HttpHeaders(headers) };
        this.serviwRest.settings.httpOptions = data;
        return !!token;
    }
    /**
     * Resets user
     */
    resetUser() {
        UserSubjec.next({ isExpiredToken });
        localStorage.removeItem(this.keyNameToken);
        this.configToken('');
        this.resetTimer();
        this.redirectLogin();
    }
    /**
     * Resets timer
     */
    resetTimer() {
        if (this.timerRefreshToken) {
            clearInterval(this.timerRefreshToken);
            this.timerRefreshToken = null;
        }
    }
    /**
     * Gets user
     * @returns user
     */
    getUser() {
        const user = UserSubjec.value;
        // user in service
        if (user[this.authOptions.userIdentityField])
            return user;
        // user not in localstorage
        const token = this.getToken();
        if (this.authOptions.token && !token[this.authOptions.token?.fieldName])
            return { isExpiredToken };
        return this.refactorJtwUser(token);
    }
    /**
     * Gets token
     * @returns
     */
    getToken(error = true) {
        const token = JSON.parse(localStorage.getItem(this.keyNameToken) || '{}');
        if (typeof token != 'object' || !this.authOptions.token || !token[this.authOptions.token?.fieldName]) {
            if (error)
                this.serviwGrowl.error(this.keyNameToken, 'Error al deserializar el Token');
            return {};
        }
        return token;
    }
    /**
     * Decodes jwt
     * @param token
     * @returns jwt
     */
    decodeJwt(token) {
        let jwtData = token.split('.')[1];
        let decodedJwtJsonData = window.atob(jwtData);
        let user = JSON.parse(decodedJwtJsonData);
        return user;
    }
    /**
     * Determines whether expired auth is
     * @returns true if expired auth
     */
    async isExpiredAuth() {
        if (!this.init)
            return !await this.authentication();
        else if (!ServiwUtil.empty(this.getToken(false)))
            return this.getUser().isExpiredToken();
        else
            return true;
    }
    /**
     * Determines whether autenticate is
     * @returns true if autenticate
     */
    isAutenticate() {
        return !this.getUser().isExpiredToken();
    }
    /**
     * Logouts auth jtw token service
     */
    logout() {
        this.resetUser();
    }
}
/** @nocollapse */ /** @nocollapse */ AuthJtwTokenService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: AuthJtwTokenService, deps: [{ token: ServiwRest }, { token: ServiwGrowl }, { token: i3.Router }, { token: SERVIW_AUTH_OPTIONS }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ AuthJtwTokenService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: AuthJtwTokenService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: AuthJtwTokenService, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }], ctorParameters: function () { return [{ type: ServiwRest }, { type: ServiwGrowl }, { type: i3.Router }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [SERVIW_AUTH_OPTIONS]
                }] }]; } });

// ----------- @Inject(<NAME>)
const SERVIW_AUTH_SERVICE = new InjectionToken('Serviw Auth service');
const SERVIW_AUTH_OPTIONS = new InjectionToken('Serviw Auth options');
// ----------- AUTH
const ValidationEmail = /^[a-zA-Z0-9]+$/;
/**
 * @param username
 * @returns AuthOptions
 */
const AuthOptionsDefault = (username = 'username') => {
    return {
        service: AuthJtwTokenService,
        userIdentityField: username,
        validations: {
            username: { pattern: ValidationEmail, nullable: false, minLength: 1, maxLength: 32 },
            password: { nullable: false, minLength: 0, maxLength: 32 }
        },
        login: {
            endpoint: 'login_check',
            messageError: 'Nombre de usuario o contraseña incorrecta.',
            messageSuccess: 'Bienvenido {name}',
            redirectSuccess: '/',
            path: '/login'
        },
        authentication: {
            endpoint: AuthJWTOptionsDefault.endpointRefresh,
            fieldInUser: 'authentication',
            messageError: 'Error en la autenticación.',
            callAfterLogin: false
        },
        logout: {
            endpoint: 'logout',
            messageError: 'Error al cerrar la sesión.',
        },
        token: AuthJWTOptionsDefault
    };
};

class AuthService {
    /**
     * Creates an instance of auth service.
     * @param authInterface
     */
    constructor(authInterface) {
        this.authInterface = authInterface;
        if (null == authInterface)
            throw Error('No se ha  implementado el módulo ServiwAuthModule.forRoot()');
    }
    /** Gets auth options */
    get authOptions() { return this.authInterface.authOptions; }
    ;
    /** redirectSuccess */
    get redirectSuccess() { return this.authOptions?.login.redirectSuccess; }
    /**
     * Login auth service
     */
    login(data) {
        return this.authInterface.login(data);
    }
    getUser() {
        return this.authInterface.getUser();
    }
    authentication() {
        return this.authInterface.authentication();
    }
    resetUser() {
        return this.authInterface.resetUser();
    }
    isExpiredAuth() {
        return this.authInterface.isExpiredAuth();
    }
    isAutenticate() {
        return this.authInterface.isAutenticate();
    }
    logout() {
        return this.authInterface.logout();
    }
}
/** @nocollapse */ /** @nocollapse */ AuthService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: AuthService, deps: [{ token: SERVIW_AUTH_SERVICE }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ AuthService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: AuthService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: AuthService, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }], ctorParameters: function () { return [{ type: AuthServiceInterface, decorators: [{
                    type: Inject,
                    args: [SERVIW_AUTH_SERVICE]
                }] }]; } });

class ServiwLogin {
    constructor(authService, router, serviwFormService) {
        this.authService = authService;
        this.router = router;
        this.serviwFormService = serviwFormService;
        this.hide = true;
        const validations = authService.authOptions?.validations;
        if (validations) {
            this.form = new FormGroup({
                username: new FormControl(null, [
                    this.serviwFormService.validatorOrOpts({ validations: validations.username }),
                ]),
                password: new FormControl(null, [
                    this.serviwFormService.validatorOrOpts({ validations: validations.password })
                ]),
            });
        }
    }
    /**
     * Hides icon
     */
    hideIcon() {
        this.hide = !this.hide;
    }
    /**
     * Logins login component
     */
    async login() {
        this.loading = true;
        const user = await this.authService.login(this.form?.value);
        await (this.loading = false);
        if (!await ServiwUtil.empty(user) && this.authService.redirectSuccess)
            this.router.navigateByUrl(this.authService.redirectSuccess);
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwLogin.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLogin, deps: [{ token: AuthService }, { token: i3.Router }, { token: ServiwFormService }], target: i0.ɵɵFactoryTarget.Component });
/** @nocollapse */ /** @nocollapse */ ServiwLogin.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.1.2", type: ServiwLogin, selector: "app-login", ngImport: i0, template: '', isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwLogin, decorators: [{
            type: Component,
            args: [{
                    selector: 'app-login',
                    template: '',
                }]
        }], ctorParameters: function () { return [{ type: AuthService }, { type: i3.Router }, { type: ServiwFormService }]; } });

class ServiwAuthModule {
    static forRoot(options) {
        if (!options)
            options = AuthOptionsDefault();
        return {
            ngModule: ServiwAuthModule,
            providers: [
                { provide: SERVIW_AUTH_SERVICE, useClass: options.service },
                { provide: SERVIW_AUTH_OPTIONS, useValue: { ...options, service: null } }
            ]
        };
    }
}
/** @nocollapse */ /** @nocollapse */ ServiwAuthModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwAuthModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ /** @nocollapse */ ServiwAuthModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwAuthModule, declarations: [ServiwLogin], imports: [CommonModule], exports: [ServiwLogin] });
/** @nocollapse */ /** @nocollapse */ ServiwAuthModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwAuthModule, providers: [], imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.2", ngImport: i0, type: ServiwAuthModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [ServiwLogin],
                    exports: [ServiwLogin],
                    providers: [],
                }]
        }] });

/*
 * Public API Surface of serviw-core
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AuthJWTOptionsDefault, AuthJtwTokenService, AuthOptionsDefault, AuthService, AuthServiceInterface, CoreProperty, CorePropertyGraphql, CorePropertyRest, EntityGraphql, EntityRest, ErrorException, INPUTNUMBER_VALUE_ACCESSOR, InputNumber, InputNumberModule, JSON_STORAGE, MANY_TO_MANY, MANY_TO_ONE, ONE_TO_MANY, ONE_TO_ONE, OPEN_SERVIW_FORM, SCHEMA_GRAPHQL, SCHEMA_REST, SERVIW_AUTH_OPTIONS, SERVIW_AUTH_SERVICE, SERVIW_LOADING, SERVIW_USER, ServiwAuthModule, ServiwBuilder, ServiwControl, ServiwCoreForm, ServiwCoreJson, ServiwCoreModule, ServiwCoreTable, ServiwEvents, ServiwForm, ServiwFormControl, ServiwFormDirective, ServiwFormField, ServiwFormService, ServiwGraphqlService, ServiwGrowl, ServiwJsonStorage, ServiwLoading, ServiwLoadingComponent, ServiwLoadingModule, ServiwLogin, ServiwParams, ServiwQueryService, ServiwRest, ServiwUtil, ServiwValidator, SettingsServiwCore, SvwJsonEvents, TYPES, UserSubjec, ValidationEmail, getExpiredToken, getNow, isExpiredToken, queryTokens, userObservable };
//# sourceMappingURL=serviw-core.mjs.map
