import { Router } from "@angular/router";
import { ServiwGrowl } from "../../services/serviw-growl.service";
import { ServiwRest } from "../../services/serviw-rest";
import { AuthServiceInterface } from "../auth-service.interface";
import { AuthOptions } from "../auth-options";
import { JwtUser } from "./jwt-utils";
import * as i0 from "@angular/core";
export declare class AuthJtwTokenService<I> extends AuthServiceInterface<I, JwtUser<I>> {
    private serviwRest;
    private serviwGrowl;
    private router;
    readonly authOptions: AuthOptions;
    /** Timer refresh token of auth service */
    private timerRefreshToken;
    /** keyNameToken */
    readonly keyNameToken: string;
    /** Init */
    init: boolean;
    /**
     * Creates an instance of user service.
     * @param serviwRest
     * @param serviwGrowl
     */
    constructor(serviwRest: ServiwRest, serviwGrowl: ServiwGrowl, router: Router, authOptions: AuthOptions);
    /** API */
    get api(): string;
    /** percentageToRefresh  */
    get percentRefresh(): number;
    /**
     * Redirects login
     */
    redirectLogin(): void;
    /**
     * Logins user service
     * @param username
     * @param password
     * @returns
     */
    login(data: I): Promise<JwtUser<I>>;
    /**
     * Authentications auth jtw token service
     * @template I
     * @returns authentication
     */
    authentication(): Promise<boolean>;
    /**
     * Refreshs token
     */
    refreshToken(token: any): Promise<JwtUser<I>>;
    /**
     * Loads token
     * @param observable
     * @returns
     */
    private loadToken;
    /**
     * Sets user
     * @param response
     * @returns
     */
    private refactorJtwUser;
    /**
     * Sets user
     * @param user
     */
    private setUser;
    /**
     * Tokens to refresh
     * @param token
     * @param [force]
     * @returns
     */
    tokenToRefresh(token: any, force?: boolean): boolean;
    /**
     * Configs token
     */
    private configToken;
    /**
     * Resets user
     */
    resetUser(): void;
    /**
     * Resets timer
     */
    resetTimer(): void;
    /**
     * Gets user
     * @returns user
     */
    getUser(): JwtUser<I>;
    /**
     * Gets token
     * @returns
     */
    getToken(error?: boolean): any;
    /**
     * Decodes jwt
     * @param token
     * @returns jwt
     */
    decodeJwt(token: string): JwtUser<I>;
    /**
     * Determines whether expired auth is
     * @returns true if expired auth
     */
    isExpiredAuth(): Promise<boolean>;
    /**
     * Determines whether autenticate is
     * @returns true if autenticate
     */
    isAutenticate(): boolean;
    /**
     * Logouts auth jtw token service
     */
    logout(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<AuthJtwTokenService<any>, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<AuthJtwTokenService<any>>;
}
//# sourceMappingURL=auth-jtw-token.service.d.ts.map