import { AbstractControl, ValidationErrors } from '@angular/forms';
import { ServiwControl } from './serviw-control';
export declare class ServiwValidator {
    static greaterThan(object: any, field: string): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Comparar el valor actual sea mayor o igual al valor del control
     * asignado como parametro nameControl, del FormGroup actual
     *
     * @param nameControl
     * @param add valor adiciona de diferencia
     */
    static minIn(nameControl: string, add?: number): (control: ServiwControl) => ValidationErrors | null;
    /**
     * Comparar el valor actual sea menor al valor del control
     * asignado como parametro nameControl, del FormGroup actual
     *
     * @param nameControl
     * @param add valor adiciona de diferencia
     */
    static maxIn(nameControl: string, add?: number): (control: ServiwControl) => ValidationErrors | null;
    static min(min: number): (control: AbstractControl) => ValidationErrors | null;
    static max(max: number): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Validar fecha a ingresar sea menor o igual al valor asignado por parametro
     * @param max Fecha máxima
     * @param format formato de error
     * @default format 'dd / MMM / yyyy'
     */
    static maxDate(max: Date, format?: string): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Mins date in
     * @param nameControl
     * @returns
     */
    static minDateIn(nameControl: string): (control: ServiwControl) => ValidationErrors | null;
    /**
     * Maxs date in
     * @param nameControl
     * @returns
     */
    static maxDateIn(nameControl: string, format?: string): (control: ServiwControl) => ValidationErrors | null;
    /**
     * Validar fecha a ingresar sea mayor o igual al valor asignado por parametro
     * @param min  Fecha mínima
     * @param [format] 'dd / MMM / yyyy'
     * @param [error] Fecha mínima a ingresar: {value}
     * @returns  call
     */
    static minDate(min: Date, format?: string, error?: string): (control: ServiwControl) => ValidationErrors | null;
    /**
     * Properties serviw validator
     * @param objects
     * @param error
     * @returns
     */
    static properties(objects: number, error: string): (control: AbstractControl) => ValidationErrors | null;
    static object(error: string): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Desactivar controles de un formulario si esta asignado contenido
     * al primer ingreso
     */
    static disableEdit(): (control: AbstractControl) => any;
    static nullValue(): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Consulta asíncrona, donde se valida un si existe un registro en el servidor.
     *
     * 1. Define el número de controles a suscribir
     * 2. Sección de retorno, Promise.
     *  2.1. Establecer control de validación al servidor
     *  2.2. Obtiene y asigna funciones de validación
     *
     * @param id identificador de cada valor si es objeto,
     * @param editable edita o no el con contenido
     * @requires Promise<any> in suscribeControls
     * @description se identifican propiedades, asignadas dentro de otra por un punto(.).
     * Se recomienda usar validación en los campos con @var ngIF ...control.errors para visializar errores si y solo si hay error
     */
    static validServer(id?: string): any;
    /**
     * Validar valores eliminando espacios en blanco
     */
    static blanks(): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Validar datos numericos, omitiendo valores nulos
     *
     * 1. Asignar valor ingresado, valida dato numerico
     * 2. Datos numericos invalidos
     *
     * @param element
     */
    static requiredNumber(): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Establecer una lista personalizada de valores, que se validan
     * con el valor ingresado en el control.
     *
     * 1. Tipo de retorno, en AbstractControl
     *  1.1. Declara valor y valor valido.
     *  1.2. Define si un valor coincide con el actual
     *  1.3. Establece error.
     *
     * @param vals
     */
    static validData<I>(values: I[]): (control: AbstractControl) => ValidationErrors | null;
    static equals(value: any, error?: string): (control: AbstractControl) => ValidationErrors | null;
    /**
     * Equals serviw validator
     * @param nameControl
     * @returns
     */
    static equalsIn(nameControl: string, error?: string): (control: ServiwControl) => ValidationErrors | null;
}
//# sourceMappingURL=serviw-validator.d.ts.map