import { CoreEntity } from './../world/schemas';
import { SvwProperties } from './../world/interfaces';
import { FormGroup, ValidatorFn, AbstractControl, FormControl } from '@angular/forms';
import { SvwProperty } from '../world/interfaces';
import { SvwPropertyGraphql, SvwJsonGraphql } from '../world/interfaces-graphql';
import { SvwJsonRest } from '../world/interfaces-rest';
import { SettingsServiwCore } from './settings-serviw-core';
import { Subscription } from 'rxjs';
import { ServiwBuilder, ServiwForm } from './serviw-builder.service';
import * as i0 from "@angular/core";
export declare class ServiwFormService {
    builder: ServiwBuilder;
    private settings;
    /**
     * Creates an instance of serviw form service.
     * @param builder
     */
    constructor(builder: ServiwBuilder, settings: SettingsServiwCore);
    /**
     *
     * @param type
     */
    static validType(type: string): string;
    /**
     * Refactors type
     * @param type
     * @param [forceType]
     * @returns
     */
    static refactorType(field: string, property: SvwPropertyGraphql): any;
    /**
     * Refactors set data form
     * @param value
     * @param properties
     * @param [formGroup] set values null in fields not in value
     * @returns set data form
     */
    static refactorSetDataForm<I = any>(value: any, properties: SvwProperties, formGroup?: FormGroup): I;
    /**
     * Gets form json
     * @param json
     * @param [value]
     * @returns form json
     */
    getFormJson<I>(json: SvwJsonRest<I> | SvwJsonGraphql<I>, value?: I): {
        form: FormGroup;
        entity: CoreEntity<I>;
    };
    /**
     * Gets form
     * @param json
     * @param [subscribe]
     * @returns form
     */
    getForm<I>(json: CoreEntity<I>, subscribe?: boolean): ServiwForm<I>;
    /**
     * Configs property
     * @param property
     * @param name
     * @returns
     */
    private configProperty;
    /**
     * Validations serviw form service
     * @param property
     * @returns validation
     */
    validation(property: SvwProperty): FormControl;
    /**
     * Set validator for control
     * @param property
     */
    validatorOrOpts(property: SvwProperty): ValidatorFn | null;
    validations(): void;
    /**
     * Suscribir controles al cambio de valores, asignando el detalle de cada error
     *
     * 1. Selecciona los controles actuales
     * 2. Callback ejecutado al cambio de valores
     * 3. Validar cada control
     *
     * @param formGroup Formulario a suscribir
     * @description Se asigna como primer campo, nombre de la
     * variable en el servicio Links, y una ruta consecutiva, como opción
     * que se agrega a la ruta definida.
     *
     * Se reconoce el tipo de petición count o get, según la ultima palabra del link
     *
     * @param formGroup
     * @param [call]
     * @returns subscription
     * @example { CostoUnitarioPeriodoFormComponent#loadForm() }
     */
    suscribeControls<T>(formGroup: FormGroup, call?: (next: T) => void): Subscription;
    /**
     * Suscribes control
     * @param control
     * @param [name]
     * @returns  control
     */
    suscribeControl(control: AbstractControl, name?: string): AbstractControl;
    /**
     * Establecer mensajes personalizados a errores establecidos en el formulario
     *
     * 1. Selecciona errores de un control
     * 2. Establece nombre del control
     * 3. Asignar detalle de cada error
     *
     * @param control control a validar errores
     * @param nameControl nombre del control
     */
    private setErrors;
    /**
     * Refactors data form
     * @param value
     * @param entity
     * @returns data form
     */
    static refactorDataForm<I>(value: any, entity: CoreEntity<I>): any;
    /**
     * Refactors array value
     * @param array
     * @param entity
     * @returns
     */
    private static refactorArrayValue;
    static ɵfac: i0.ɵɵFactoryDeclaration<ServiwFormService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<ServiwFormService>;
}
//# sourceMappingURL=serviw-form.service.d.ts.map