import { GraphqlBody, GraphqlFilters, SvwPropertyGraphql, SvwJsonGraphql, GraphqlFilter } from './../world/interfaces-graphql';
import { FormGroup } from '@angular/forms';
import { EntityGraphql } from '../classes/entity-graphql';
import { ServiwRest } from './serviw-rest';
import { SettingsServiwCore } from './settings-serviw-core';
import { TypeProperties, RefactorQuery, QuerySingle, SvwProperty } from '../world/interfaces';
import { EntitySave } from '../world/interfaces-graphql';
import * as i0 from "@angular/core";
export declare const queryTokens: {
    separators: string[];
    separatorTokens: string[];
    openBrace: string;
    closeBrace: string;
};
export declare class ServiwGraphqlService {
    serviwRest: ServiwRest;
    settings: SettingsServiwCore;
    /**
     * Determines whether save in
     */
    private inSave;
    /**
     * Creates an instance of serviw query service.
     * @param http
     * @param settings
     */
    constructor(serviwRest: ServiwRest, settings: SettingsServiwCore);
    /**
     * Gets rest
     */
    get rest(): ServiwRest;
    /**
     * Excludes fields graphql
     * @param field
     * @returns fields graphql
     */
    excludeFieldsGraphql(field: string): boolean;
    /**
     * Querys save
     *
     * Define data query for schema
     *
     * 1. Define functions response
     * 2. Callback update Form
     * 3. Data for schema Graphql
     * 4. Data for schema REST-POST
     *
     * @param entity Entity
     * @param formGroup FormGroup
     * @returns EntitySave
     */
    querySave<I>(entity: EntityGraphql<I>, formGroup: FormGroup): EntitySave;
    /**
     * Querys graphql text
     * @param table
     * @param paramsTypesValues sintax nameParam:type:value
     */
    queryGraphqlSyntax(table: string, paramsTypesValues: string): RefactorQuery;
    /**
     * Gets params fields
     * @param paramsTypesValues
     * @returns
     */
    private getParamsFields;
    /**
     * Querys single graphql
     * @param entity name
     * @param fields sintax id,name,...
     * @param valueId value
     * @param [nameId] id
     * @returns  query | refactor | post
     */
    querySingleGraphql<I>(entity: string, fields: string, valueId: string, nameId?: string): QuerySingle<I>;
    /**
     * Querys delete graphql
     * @param entity
     * @param fields
     * @param valueId
     * @param [nameId]
     * @returns delete graphql
     */
    queryDeleteGraphql<I>(entity: string, fields: string, valueId: string, nameId?: string): QuerySingle<I>;
    /**
     * Separate keys for graphql queries
     * @param keys for separate
     * @returns keys separate in array
     */
    separateKeys(keys: string): string[];
    /**
     * Querys graphql
     *
     * 1. Name Entity
     * 2. Define schema Graphql types, params
     * 3. Define fields of query
     * 4. Set variables, create graphqlBody
     * 5. refactor data response
     * 6. refactor data response in options
     *
     * @param table name Entity
     * @param params data query
     * @param fields response
     * @param [paginator] paginator
     * @returns query | refactor | getOptions
     */
    queryGraphql(table: string, params: GraphqlFilters, fields: string, paginator?: string): RefactorQuery;
    /**
     * Refactors edges->nodes
     *
     * 1. get edges
     * 2. Valid Array
     * 3. refactor, edge for node
     *
     * @param data response
     * @param keys name properties
     * @returns
     */
    refactorEdges(data: any, keys: string[]): {}[];
    /**
     * Update formulario in the response the server
     * @param response data
     * @param formGroup setValue
     * @param json Entity
     * @param [operationName] acción realizada create<Table>, update<Table>
     * @returns
     */
    updateFormGroup<I>(response: any, formGroup: FormGroup, json: SvwJsonGraphql<I>, operationName?: string): any;
    /**
     * Define the scheme to save or edit the data of an entity
     *
     * 1. Valid data id
     * 2. Deefine create or update
     * 3. Create GraphqlBody
     *
     * @param entity Entity
     * @param formGroup FormGroup
     * @returns graphqBody
     */
    queryAddEditGraphql<I>(entity: EntityGraphql<I>, formGroup: FormGroup): GraphqlBody;
    /**
     * Create queries to save or edit in the graphql schema
     *
     * 1. Valid save or edit
     * 2. Structure  of query
     * 3. Replace types, params, fields
     *
     * @param entity Entity
     * @param operationName action
     * @param [empyId] add or edit
     * @returns save graphql
     */
    private querySaveGraphql;
    /**
     * Gets types graphql
     *
     * 1. Get fields EntityJSON
     * 2. Valid empyId
     * 3. Define name and type of properties
     * 4. Remove ', '
     *
     * @param entity Entity
     * @param [empyId] add or edit
     * @returns types graphql
     */
    private getTypesGraphql;
    /**
     * Gets params graphql
     *
     * 1. Get fields EntityJSON
     * 2. Valid empyId
     * 3. Define properties
     * 4. Remove ', '
     *
     * @param entity entity
     * @param [empyId]  add or edit
     * @returns params graphql
     */
    private getParamsGraphql;
    /**
     * Skips write on mutation input
     * @param field
     * @param property
     * @returns true if write on mutation input
     */
    private skipWriteOnMutationInput;
    /**
     * Gets fields graphql
     *
     * 1. fields storage
     * 2. Set fields for each property
     *
     * @param properties fields
     * @returns fields graphql
     */
    getFieldsGraphql<I>(properties: TypeProperties<I, SvwPropertyGraphql>): string;
    /**
     * Gets field filter
     * @template S
     * @param property
     * @param value
     * @param [forceType]
     * @returns field filter
     */
    static getFieldFilter<S>(property: SvwProperty<S>, value: any, forceType?: boolean): GraphqlFilter;
    static ɵfac: i0.ɵɵFactoryDeclaration<ServiwGraphqlService, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<ServiwGraphqlService>;
}
//# sourceMappingURL=serviw-graphql.service.d.ts.map