import { LazyLoadEvent } from './../world/interfaces';
import { Observable } from 'rxjs';
import { HttpClient, HttpParams, HttpHeaders } from '@angular/common/http';
import { SettingsServiwCore } from './settings-serviw-core';
import * as i0 from "@angular/core";
/**
 * Filters load
 */
export interface FiltersLoad {
    [name: string]: string | number | boolean;
}
/**
 * Core http params
 */
export interface CoreHttpParams {
    params?: HttpParams | {
        [param: string]: string | string[];
    };
    headers?: HttpHeaders | {
        [header: string]: string | string[];
    };
}
/**
 * Core http options
 */
export interface CoreHttpOptions {
    headers?: HttpHeaders | {
        [header: string]: string | string[];
    };
    observe?: 'body';
    params?: HttpParams | {
        [param: string]: string | string[];
    };
    reportProgress?: boolean;
    responseType: 'arraybuffer' | 'blob' | 'json' | 'text';
    withCredentials?: boolean;
}
/**
 * Core refactor
 */
export interface CoreRefactor<T, R> {
    (data: T): R;
}
export declare class ServiwRest {
    http: HttpClient;
    settings: SettingsServiwCore;
    /**
     * Get http of serviw rest
     */
    private _getHttp;
    /**
     * Creates an instance of serviw rest.
     * @param http
     */
    constructor(http: HttpClient, settings: SettingsServiwCore);
    /**
     * Gets options
     */
    get options(): CoreHttpOptions;
    /**
     * Get Observable http query
     */
    get getHttp(): Observable<"filter" | "get" | "post" | "put" | "delete" | "countFilter">;
    /**
     * Gets serviw rest
     * @param link
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns get
     */
    get<I, R>(link: string, params?: FiltersLoad, refactor?: CoreRefactor<I, R>, options?: CoreHttpOptions): Promise<R>;
    /**
     * Posts serviw rest
     * @param link
     * @param body
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns
     */
    post<I, R>(link: string, body: any, params?: FiltersLoad, refactor?: CoreRefactor<I, R>, options?: CoreHttpOptions): Promise<R>;
    /**
     * Puts serviw rest
     * @param link
     * @param body
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns
     */
    put<I, R>(link: string, body: any, params?: FiltersLoad, refactor?: CoreRefactor<I, R>, options?: CoreHttpOptions): Promise<R>;
    /**
     * Deletes serviw rest
     * @param link
     * @param [params]
     * @param [refactor]
     * @param [options]
     * @returns
     */
    delete<I, R>(link: string, params?: FiltersLoad, refactor?: CoreRefactor<I, R>, options?: CoreHttpOptions): Promise<R>;
    /**
     * Refactors observable
     * @param observable
     * @param refactor
     * @returns observable
     */
    refactorObservable<I, R>(observable: Observable<any>, refactor?: CoreRefactor<I, R>): Promise<R>;
    /**
     * Refactors data
     * @param response
     * @param refactor
     * @returns data
     */
    private refactorData;
    /**
     * Gets link parameters
     * @param data
     * @returns link parameters
     */
    getLinkParameters(data: FiltersLoad): string;
    /**
   * Filters serviw rest
   * @param link extends GenericEntityFilter<T>.filter
   * @param event table
   * @param [omitPage] omit pagination = false
   * @returns Promise<T>
   */
    filter<T, R>(link: string, event: LazyLoadEvent, omitPage?: boolean, refactor?: (data: T) => R): Promise<R>;
    /**
     * Counts filter
     * @param link GenericEntityFilter<T>.countFilter
     * @param event
     * @returns count filter
     */
    countFilter(link: string, event: LazyLoadEvent): Promise<number>;
    /**
     * Refactors filter by
     * @param event
     * @param [omitPage]
     * @returns
     */
    refactorFilterBy(event: LazyLoadEvent, omitPage?: boolean): any;
    /**
     * Establecer parametros a filtrar en una consulta slice.
     *
     * 1. Almacena datos a filtrar
     * 2. Establece filtros asignados en una tabla
     * 3. Establece filtros personalizados
     * 4. Retorna filtros para consultas  @function count()
     * 5. Establece datos para @var PageRequest
     * 6. Define columna y el tipo a ordenar
     *
     * @param event Datos de una consulta
     * @param count Consultas count
     */
    refactorFilters(event: LazyLoadEvent, count?: boolean): FiltersLoad;
    /**
     * Filters omit
     * @param filters
     * @returns omit
     */
    private filtersOmit;
    /**
     * Gets link parameters
     * @param data
     * @returns link parameters
     */
    static getLinkParameters(data: any): string;
    /**
     * Options defult
     * @returns defult
     */
    static optionsDefult(): CoreHttpOptions;
    /**
     * Bads request
     * @returns request
     */
    static BAD_REQUEST(): number;
    static ɵfac: i0.ɵɵFactoryDeclaration<ServiwRest, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<ServiwRest>;
}
//# sourceMappingURL=serviw-rest.d.ts.map