import * as i0 from "@angular/core";
export declare class ServiwUtil {
    constructor();
    /**
     * Retornar un mes anterior al actual
     *
     * @param mes
     */
    mesAnterior(mes: number): number;
    /**
     * Calcula y retorna un año anterior, según el mes
     *
     * @param ano
     * @param mes
     */
    anoAnterior(ano: number, mes: number): number;
    /**
     * Asignar datos de un array a una interfaz dada, donde distribuye los datos
     * en secuencia de array a la interfaz.
     *
     * 1. Recorre cada tipo de dato en interfaz
     *  1.1. Obtiene el valor y el tipo de dato
     *  1.2. Refactoriza objects de la interfaz
     *  1.3. Establece el valor asignado
     *
     * @param interfaz Object parametriza tipo y secuencia de datos
     * @param data Datos a asignar en la interfaz
     * @param i Inicio lectura de datos en array
     * @default i = 0
     */
    refactorInterfaz<I>(interfaz: I, data: any[], i?: number): number;
    /**
     * Define el tipo de dato a retornar
     *
     * 1. Define el tipo de dato
     * 2. Realiza la conversión al dato asignado
     *
     * @param data Dato a convertir
     * @param type type Define el tipo de dato a retornar
     * @description Date | Object | string | number | boolean, Donde se asigna
     * Cualquier valor en (type) que define el tipo de dato a retornar
     * @returns data
     */
    getDataType<I>(data: any, type: I): I;
    /**
     * Determines if value is objet
     * @param value to evaluate
     * @returns
     */
    static isObject(value: any): boolean;
    static isString(value: any): boolean;
    static isNumber(value: any): boolean;
    /**
     * Clones object
     * @param object
     * @returns object
     */
    static cloneObject<I extends object>(object: I): I;
    /**
     * Validar si los valores de una propiedad son nulos o indefinidos.
     * También valida si el numero de propiedades corresponde al número
     * asignado en parámetro.
     *
     * 1. Valida numero de propiedades
     * 2. Valida el valor de cada propiedad
     *
     * @param object interfaz a validar
     * @param numbers numero de propiedades en interfaz original
     */
    static validPropierties<Interfaz>(object: Interfaz, numbers?: number): boolean;
    /**
     * Emitir cambios de una suscripción #addEditCall para actualizar o agregar
     * un objeto en una lista (Array)
     *
     * 1. Validar elementos de la suscripción
     * 2. Declarar variables
     * 3. Recorre cada dato de un array
     *  3.1. Compara identificadores de cada objeto
     *  3.2. Actualiza un valor de la tabla
     *  3.3. Agrega valores actuales
     * 4. Agrega un nuevo elemento
     * 5. Emitir valores y reseter datos
     *
     * @param array Array
     * @param addUpdate Objecto a actualizar
     * @param id  @default{id} {Id} Identificador
     * @param esquemaId Propiedades a comparar id tipo object
     * @param remove Dato/s a remover en un array
     * @param unshift Agregar items al inicio o final
     */
    static updateArray<I>(array: I[], addUpdate: I | I[], id?: string, esquemaId?: any, remove?: I | I[], unshift?: boolean): I[];
    /**
     * Remover uno o mas elementos de un array
     *
     * 1. Validar elementos a remover
     * 2. Define array a remover
     * 3. Elimina cada elemento de la lista
     *
     * @param remove Elemento o lista a remover
     * @param vals Array de consulta
     * @param id Identificador para objetcs
     * @param esquemaId Esquema Id personalizado
     */
    static removeArray<Interfaz>(remove: Interfaz | Interfaz[], vals: Interfaz[], id?: string, esquemaId?: any): Interfaz[];
    /**
     * Consultar index de un elemento en un array
     *
     * @param val Elemento en array
     * @param vals Array de consulta
     * @param id Identificador para objetcs
     * @param esquemaId Esquema Id personalizado
     */
    static searchIndex<Interfaz>(val: Interfaz, vals: Interfaz[], id?: string, esquemaId?: any): number;
    /**
     * Validar si dos objetos son iguales.
     *
     * 1. Validar datos diferentes a nulos
     * 2. Interfaz Datos a comparar
     * 3. Establecer datos por esquema personalziado
     * 4. Comparar objetos
     *
     * @param obj1 objeto a comparar
     * @param obj2 objeto a comparar
     * @param esquemaId Esquema personalizado a comparar
     */
    static objEquals<K, I>(obj1: K, obj2: I | I, esquema?: I | K): boolean;
    /**
     * Logs serviw util
     * @param data
     */
    static logs(...data: any): void;
    /**
     * Clonar datos de un objeto (data) hacia un esquema (key)
     *
     * 1. Inicializar datos de respuesta
     * 2. Establecer datos por propiedad
     *
     * @param key Esquema a clonar de data
     * @param data Objecto original
     */
    static getEsquema<K, I>(key: K, data: I): K;
    /**
     * Emptys basic
     * @param data
     * @returns true if basic
     */
    static isEmpty(data: any): boolean;
    /**
     * Params serviw util
     * @param data validate
     * @param [omit] data == value  return false
     * @returns true if data = ( null || undefined || '' || {} || [] || !data )
     * @description || -> o
     */
    static empty(data: any, omit?: any): boolean;
    /**
     * Valids date
     * @param date
     * @param [today]
     * @returns date of date
     */
    static validDate(date: any, today?: boolean): Date;
    /**
     * Implement datePipe
     *
     * @param value
     * @param format
     * @param timezone
     * @param locale
     */
    static datePipe(value: any, format?: string, timezone?: string, locale?: string): string | null;
    /**
     * Consultar el valor de la ultima propiedad asignada en un object
     *
     * 1. Obtener copias del objeto actal
     * 2. Separar las propiedades por caracter (.)
     * 3. Consulta y filtra datos de las propiedades
     *
     * @param filter
     */
    static valueProperties<I>(item: any, properties: string): I;
    /**
     *
     * @param quantity
     */
    static numberLength(quantity: number): number;
    /**
     * Primera letra a mayuscula
     * @param text
     */
    static firstCapitalLetter(text: string): string;
    /**
     * Primera letra a mayuscula
     * @param text
     */
    static firstLowercaseLetter(text: string): string;
    /**
     * Sets key value
     * @param object
     * @param keyValue
     * @returns key value
     */
    static setKeyValue(object: any, keyValue: string): Object;
    /**
     * Resets object
     * @param obj
     */
    static resetObject(obj: any): Object;
    /**
     * Returns serviw util
     * @returns object {}
     */
    static emptyObject(): any;
    static ɵfac: i0.ɵɵFactoryDeclaration<ServiwUtil, never>;
    static ɵprov: i0.ɵɵInjectableDeclaration<ServiwUtil>;
}
//# sourceMappingURL=serviw-util.service.d.ts.map