import { SvwValidators, SvwProperty, TypeProperties, SvwJson } from './interfaces';
export declare type Primitive = number | boolean | string;
/**
 * Svw entity graphql
 */
export interface SvwJsonGraphql<I, S = any> extends SvwJson {
    singular?: string;
    plural?: string;
    fields?: TypeProperties<I, SvwPropertyGraphql<S>>;
    filters?: {
        [P in Extract<keyof I, string>]?: GraphqlFilter | Primitive;
    };
}
/**
 * Svw property graphql
 */
export interface SvwPropertyGraphql<S = any> extends SvwProperty<S> {
    forceType?: boolean;
    /**
     * Ej: id, name, ...
     */
    propertiesGet?: string;
    validations?: SvwValidators;
}
/**
 * Graphql body
 */
export interface GraphqlBody {
    query?: string;
    variables?: {
        [name: string]: any;
    };
}
/**
 * Graphql filters
 */
export interface GraphqlFilters {
    [name: string]: GraphqlFilter;
}
/**
 * Graphql filter
 */
export interface GraphqlFilter {
    type: string;
    id?: string;
    value?: string | number | boolean;
    validations?: SvwValidators;
    forceType?: boolean;
}
/**
 * Entity save
 */
export interface EntitySave {
    data?: GraphqlBody;
    updateForm?: (response: any) => any;
    /** @deprecated implement getError */
    getErrors?: (response: any) => any[];
    getError?: (response: any) => string;
    http?: (refactor?: (data: any) => any) => Promise<any>;
}
//# sourceMappingURL=interfaces-graphql.d.ts.map